import sys
from _typeshed import StrPath
from collections.abc import Mapping

LC_CTYPE: int
LC_COLLATE: int
LC_TIME: int
LC_MONETARY: int
LC_NUMERIC: int
LC_ALL: int
CHAR_MAX: int

def setlocale(category: int, locale: str | None = None, /) -> str: ...
def localeconv() -> Mapping[str, int | str | list[int]]: ...

if sys.version_info >= (3, 11):
    def getencoding() -> str: ...

def strcoll(os1: str, os2: str, /) -> int: ...
def strxfrm(string: str, /) -> str: ...

# native gettext functions
# https://docs.python.org/3/library/locale.html#access-to-message-catalogs
# https://github.com/python/cpython/blob/f4c03484da59049eb62a9bf7777b963e2267d187/Modules/_localemodule.c#L626
if sys.platform != "win32":
    LC_MESSAGES: int

    ABDAY_1: int
    ABDAY_2: int
    ABDAY_3: int
    ABDAY_4: int
    ABDAY_5: int
    ABDAY_6: int
    ABDAY_7: int

    ABMON_1: int
    ABMON_2: int
    ABMON_3: int
    ABMON_4: int
    ABMON_5: int
    ABMON_6: int
    ABMON_7: int
    ABMON_8: int
    ABMON_9: int
    ABMON_10: int
    ABMON_11: int
    ABMON_12: int

    DAY_1: int
    DAY_2: int
    DAY_3: int
    DAY_4: int
    DAY_5: int
    DAY_6: int
    DAY_7: int

    ERA: int
    ERA_D_T_FMT: int
    ERA_D_FMT: int
    ERA_T_FMT: int

    MON_1: int
    MON_2: int
    MON_3: int
    MON_4: int
    MON_5: int
    MON_6: int
    MON_7: int
    MON_8: int
    MON_9: int
    MON_10: int
    MON_11: int
    MON_12: int

    CODESET: int
    D_T_FMT: int
    D_FMT: int
    T_FMT: int
    T_FMT_AMPM: int
    AM_STR: int
    PM_STR: int

    RADIXCHAR: int
    THOUSEP: int
    YESEXPR: int
    NOEXPR: int
    CRNCYSTR: int
    ALT_DIGITS: int

    def nl_langinfo(key: int, /) -> str: ...

    # This is dependent on `libintl.h` which is a part of `gettext`
    # system dependency. These functions might be missing.
    # But, we always say that they are present.
    def gettext(msg: str, /) -> str: ...
    def dgettext(domain: str | None, msg: str, /) -> str: ...
    def dcgettext(domain: str | None, msg: str, category: int, /) -> str: ...
    def textdomain(domain: str | None, /) -> str: ...
    def bindtextdomain(domain: str, dir: StrPath | None, /) -> str: ...
    def bind_textdomain_codeset(domain: str, codeset: str | None, /) -> str | None: ...



from nwon_django_toolbox.permissions.backend.allow_all_users_model_backend_including_inactive import (
    AllowAllUsersModelBackendIncludingInactive,
)
from nwon_django_toolbox.permissions.initial_permissions.create_permissions_for_group_mapping import (
    create_permissions_for_group_mapping,
)
from nwon_django_toolbox.permissions.permissions.is_active import IsActive
from nwon_django_toolbox.permissions.permissions.is_authenticated import IsAuthenticated
from nwon_django_toolbox.permissions.permissions.is_authenticated_and_active import (
    IsAuthenticatedAndActive,
)

__all__ = [
    "AllowAllUsersModelBackendIncludingInactive",
    "IsActive",
    "IsAuthenticated",
    "IsAuthenticatedAndActive",
    "create_permissions_for_group_mapping",
]



from typing import List

from django.utils.html import format_html


def render_list_in_admin(
    list_to_display: List[str], sort_alphabetically: bool = True
) -> str:
    """Renders a list in a Django admin table"""

    if sort_alphabetically:
        list_to_display = sorted(list_to_display)

    elements = [f"<li>{element}</li>" for element in list_to_display]
    return format_html("<ul>" + "".join(elements) + "</ul>")



from datetime import datetime
from typing import (
    Any,
    Dict,
    Generic,
    Iterable,
    List,
    Mapping,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)

import billiard
import celery
import kombu
from celery import canvas
from celery.app.base import Celery
from celery.backends.base import Backend
from celery.canvas import Signature, xmap, xstarmap
from celery.exceptions import Retry
from celery.result import EagerResult
from celery.utils.threads import _LocalStack
from celery.worker.request import Request
from typing_extensions import Literal, ParamSpec

_P = ParamSpec("_P")
_R = TypeVar("_R")

class Task(Generic[_P, _R]):
    name: str
    typing: bool
    max_retries: Optional[int]
    default_retry_delay: int
    rate_limit: Optional[str]
    ignore_result: bool
    trail: bool
    send_events: bool
    store_errors_even_if_ignored: bool
    serializer: str
    time_limit: Optional[int]
    soft_time_limit: Optional[int]
    autoregister: bool
    track_started: bool
    acks_late: bool
    acks_on_failure_or_timeout: bool
    reject_on_worker_lost: bool
    throws: Tuple[Type[Exception], ...]
    expires: Optional[Union[float, datetime]]
    priority: Optional[int]
    resultrepr_maxsize: int
    request_stack: _LocalStack
    abstract: bool
    @classmethod
    def bind(cls, app: Celery) -> Celery: ...
    @classmethod
    def on_bound(cls, app: Celery) -> None: ...
    @property
    def app(self) -> Celery: ...
    @classmethod
    def annotate(cls) -> None: ...
    @classmethod
    def add_around(cls, attr: str, around: Any) -> None: ...
    # TODO(sbdchd): might be able to use overloads to handle the case where
    # `bind=True` passes in the first argument.
    def __call__(self, *args: _P.args, **kwargs: _P.kwargs) -> _R: ...
    def run(self, *args: _P.args, **kwargs: _P.kwargs) -> _R: ...
    def start_strategy(self, app: Celery, consumer: Any, **kwargs: Any) -> Any: ...
    def delay(
        self, *args: _P.args, **kwargs: _P.kwargs
    ) -> celery.result.AsyncResult[_R]: ...
    def apply_async(
        self,
        args: Optional[Tuple[Any, ...]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        task_id: Optional[str] = ...,
        producer: Optional[kombu.Producer] = ...,
        link: Optional[Union[Signature[Any], List[Signature[Any]]]] = ...,
        link_error: Optional[Union[Signature[Any], List[Signature[Any]]]] = ...,
        shadow: Optional[str] = ...,
        *,
        # options
        countdown: float = ...,
        eta: datetime = ...,
        expires: Union[float, datetime] = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: Union[str, kombu.Queue] = ...,
        exchange: Union[str, kombu.Exchange] = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: Dict[str, str] = ...,
        ignore_result: bool = ...,
        time_limit: int = ...,
        soft_time_limit: int = ...,
    ) -> celery.result.AsyncResult[_R]: ...
    def shadow_name(
        self, args: Tuple[Any, ...], kwargs: Dict[str, Any], options: Dict[str, Any]
    ) -> None: ...
    def signature_from_request(
        self,
        request: Optional[Request] = ...,
        args: Tuple[Any, ...] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        queue: Optional[str] = ...,
        **extra_options: Any,
    ) -> Signature[_R]: ...
    subtask_from_request = signature_from_request  # XXX compat
    def retry(
        self,
        args: Optional[Tuple[Any, ...]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        exc: Optional[Exception] = ...,
        throw: bool = ...,
        eta: Optional[datetime] = ...,
        countdown: Optional[float] = ...,
        max_retries: Optional[int] = ...,
        *,
        # options
        task_id: Optional[str] = ...,
        producer: Optional[kombu.Producer] = ...,
        link: Optional[Union[Signature[Any], List[Signature[Any]]]] = ...,
        link_error: Optional[Union[Signature[Any], List[Signature[Any]]]] = ...,
        shadow: Optional[str] = ...,
        expires: Union[float, datetime] = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: Union[str, kombu.Queue] = ...,
        exchange: Union[str, kombu.Exchange] = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: Dict[str, str] = ...,
        ignore_result: bool = ...,
        time_limit: int = ...,
        soft_time_limit: int = ...,
    ) -> Retry: ...
    def apply(
        self,
        args: Optional[Tuple[Any, ...]] = ...,
        kwargs: Optional[Dict[str, Any]] = ...,
        link: Optional[Union[Signature[Any], List[Signature[Any]]]] = ...,
        link_error: Optional[Union[Signature[Any], List[Signature[Any]]]] = ...,
        task_id: Optional[str] = ...,
        retries: Optional[int] = ...,
        throw: Optional[bool] = ...,
        logfile: Optional[str] = ...,
        loglevel: Optional[str] = ...,
        headers: Optional[Mapping[str, str]] = ...,
        *,
        # options
        ignore_result: bool = ...,
        exchange: str = ...,
        routing_key: str = ...,
        priority: int = ...,
    ) -> EagerResult[_R]: ...
    def AsyncResult(
        self, task_id: str, **kwargs: Any
    ) -> celery.result.AsyncResult[_R]: ...
    def signature(
        self, args: Optional[Tuple[Any, ...]] = ..., *starargs: Any, **starkwargs: Any
    ) -> Signature[_R]: ...
    def subtask(
        self, args: Optional[Tuple[Any, ...]] = ..., *starargs: Any, **starkwargs: Any
    ) -> Signature[_R]: ...
    def s(self, *args: Any, **kwargs: Any) -> Signature[_R]: ...
    def si(self, *args: Any, **kwargs: Any) -> Signature[_R]: ...
    def chunks(self, it: Iterable[Any], n: int) -> canvas.chunks: ...
    def map(self, it: Iterable[Any]) -> xmap: ...
    def starmap(self, it: Iterable[Any]) -> xstarmap: ...
    def send_event(
        self,
        type_: str,
        retry: bool = ...,
        retry_policy: Optional[Mapping[str, int]] = ...,
        **fields: Any,
    ) -> List[Tuple[object, object]]: ...
    def replace(self, sig: Signature[Any]) -> None: ...
    @overload
    def add_to_chord(
        self, sig: Signature[Any], lazy: Literal[True]
    ) -> celery.result.AsyncResult[Any]: ...
    @overload
    def add_to_chord(
        self, sig: Signature[Any], lazy: Literal[False] = ...
    ) -> EagerResult[Any]: ...
    def update_state(
        self,
        task_id: Optional[str] = ...,
        state: Optional[str] = ...,
        meta: Optional[Dict[str, Any]] = ...,
        **kwargs: Any,
    ) -> None: ...
    def on_success(
        self, retval: Any, task_id: str, args: Tuple[Any, ...], kwargs: Dict[str, Any]
    ) -> None: ...
    def on_retry(
        self,
        exc: Exception,
        task_id: str,
        args: Tuple[Any, ...],
        kwargs: Dict[str, Any],
        einfo: billiard.einfo.ExceptionInfo,
    ) -> None: ...
    def on_failure(
        self,
        exc: Exception,
        task_id: str,
        args: Tuple[Any, ...],
        kwargs: Dict[str, Any],
        einfo: billiard.einfo.ExceptionInfo,
    ) -> None: ...
    def after_return(
        self,
        status: str,
        retval: Any,
        task_id: str,
        args: Tuple[Any, ...],
        kwargs: Dict[str, Any],
        einfo: billiard.einfo.ExceptionInfo,
    ) -> None: ...
    def add_trail(self, result: Any) -> Any: ...
    def push_request(self, *args: _P.args, **kwargs: _P.kwargs) -> None: ...
    def pop_request(self) -> None: ...
    @property
    def request(self) -> Request: ...
    @property
    def backend(self) -> Backend: ...
    @backend.setter
    def backend(self, value: Backend) -> None: ...
    @property
    def __name__(self) -> str: ...



from collections.abc import Mapping, Sequence
from types import ModuleType
from typing import Any

__all__ = [
    "_main",
    "freeze_support",
    "set_executable",
    "get_executable",
    "get_preparation_data",
    "get_command_line",
    "import_main_path",
]

WINEXE: bool
WINSERVICE: bool

def set_executable(exe: str) -> None: ...
def get_executable() -> str: ...
def is_forking(argv: Sequence[str]) -> bool: ...
def freeze_support() -> None: ...
def get_command_line(**kwds: Any) -> list[str]: ...
def spawn_main(pipe_handle: int, parent_pid: int | None = None, tracker_fd: int | None = None) -> None: ...

# undocumented
def _main(fd: int) -> Any: ...
def get_preparation_data(name: str) -> dict[str, Any]: ...

old_main_modules: list[ModuleType]

def prepare(data: Mapping[str, Any]) -> None: ...
def import_main_path(main_path: str) -> None: ...



from typing import Any

import requests  # noqa: F401
from django.db.models import QuerySet

try:
    from django.contrib.postgres import fields as postgres_fields
except ImportError:
    postgres_fields = None  # type: ignore
try:
    import coreapi
except ImportError:
    coreapi = None  # type: ignore
try:
    import uritemplate
except ImportError:
    uritemplate = None  # type: ignore
try:
    import coreschema
except ImportError:
    coreschema = None  # type: ignore
try:
    import yaml
except ImportError:
    yaml = None  # type: ignore
try:
    import requests
except ImportError:
    requests = None  # type: ignore
try:
    import pygments
except ImportError:
    pygments = None  # type: ignore
try:
    import markdown  # type: ignore
    def apply_markdown(text: str): ...

except ImportError:
    apply_markdown = None  # type: ignore
    markdown = None  # type: ignore

if markdown is not None and pygments is not None:
    from markdown.preprocessors import Preprocessor  # type: ignore

    class CodeBlockPreprocessor(Preprocessor):
        pattern: Any
        formatter: Any
        def run(self, lines: Any): ...

def pygments_css(style: Any) -> str | None: ...
def pygments_highlight(text: str, lang: str, style: Any) -> Any: ...
def md_filter_add_syntax_highlight(md: Any) -> bool: ...
def unicode_http_header(value: str | bytes) -> str: ...
def distinct(queryset: QuerySet, base: QuerySet | None) -> QuerySet: ...

SHORT_SEPARATORS: tuple[str, str]
LONG_SEPARATORS: tuple[str, str]
INDENT_SEPARATORS: tuple[str, str]

__all__ = [
    "coreapi",
    "coreschema",
    "requests",
    "postgres_fields",
    "QuerySet",
    "uritemplate",
    "yaml",
    "pygments",
    "markdown",
    "apply_markdown",
    "Preprocessor",
    "CodeBlockPreprocessor",
    "pygments_css",
    "pygments_highlight",
    "md_filter_add_syntax_highlight",
    "unicode_http_header",
    "distinct",
    "SHORT_SEPARATORS",
    "LONG_SEPARATORS",
    "INDENT_SEPARATORS",
]



from typing import Any

from django.db.models.sql import compiler
from django.db.models.sql.compiler import _AsSqlType

class SQLCompiler(compiler.SQLCompiler):
    def as_subquery_condition(self, alias: Any, columns: Any, compiler: Any) -> _AsSqlType: ...

class SQLInsertCompiler(compiler.SQLInsertCompiler, SQLCompiler): ...

class SQLDeleteCompiler(compiler.SQLDeleteCompiler, SQLCompiler):
    # https://github.com/django/django/blob/242499f2dc2bf24a9a5c855690a2e13d3303581a/django/db/backends/mysql/compiler.py#L26
    def as_sql(self) -> _AsSqlType: ...  # type: ignore[override]

class SQLUpdateCompiler(compiler.SQLUpdateCompiler, SQLCompiler): ...
class SQLAggregateCompiler(compiler.SQLAggregateCompiler, SQLCompiler): ...



from _typeshed import StrPath
from collections.abc import Generator
from typing import ClassVar
from typing_extensions import Literal

from .. import fixer_base
from ..pytree import Node

MAPPING: dict[str, str]

def alternates(members): ...
def build_pattern(mapping=...) -> Generator[str, None, None]: ...

class FixImports(fixer_base.BaseFix):
    BM_compatible: ClassVar[Literal[True]]
    mapping = MAPPING
    def build_pattern(self): ...
    def compile_pattern(self) -> None: ...
    def match(self, node): ...
    replace: dict[str, str]
    def start_tree(self, tree: Node, filename: StrPath) -> None: ...
    def transform(self, node, results) -> None: ...



from typing import Any

from ..cmd import Command

class bdist_dumb(Command):
    description: str
    user_options: Any
    boolean_options: Any
    default_format: Any
    bdist_dir: Any
    plat_name: Any
    format: Any
    keep_temp: int
    dist_dir: Any
    skip_build: Any
    relative: int
    owner: Any
    group: Any
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...



import types
from collections.abc import Iterator

from django.apps.registry import Apps
from django.db.models.base import Model
from django.utils.functional import _StrOrPromise

MODELS_MODULE_NAME: str

class AppConfig:
    name: str
    module: types.ModuleType | None
    apps: Apps | None
    label: str
    verbose_name: _StrOrPromise
    path: str
    models_module: str | None
    models: dict[str, type[Model]]
    def __init__(self, app_name: str, app_module: types.ModuleType | None) -> None: ...
    @classmethod
    def create(cls, entry: str) -> AppConfig: ...
    def get_model(self, model_name: str, require_ready: bool = ...) -> type[Model]: ...
    def get_models(self, include_auto_created: bool = ..., include_swapped: bool = ...) -> Iterator[type[Model]]: ...
    def import_models(self) -> None: ...
    def ready(self) -> None: ...



from django.contrib.admin.options import ModelAdmin
from django.db.models.query import QuerySet
from django.http.request import HttpRequest
from django.template.response import TemplateResponse

def delete_selected(modeladmin: ModelAdmin, request: HttpRequest, queryset: QuerySet) -> TemplateResponse | None: ...



import zipfile
from collections.abc import Sequence

from django.apps.config import AppConfig
from django.core.management.base import BaseCommand
from django.db.models.base import Model

READ_STDIN: str

class Command(BaseCommand):
    ignore: bool
    using: str
    app_label: str
    verbosity: int
    excluded_models: set[type[Model]]
    excluded_apps: set[AppConfig]
    format: str
    missing_args_message: str
    def loaddata(self, fixture_labels: Sequence[str]) -> None: ...
    def load_label(self, fixture_label: str) -> None: ...
    def find_fixtures(self, fixture_label: str) -> list[tuple[str, str | None, str | None]]: ...
    @property
    def fixture_dirs(self) -> list[str]: ...
    def parse_name(self, fixture_name: str) -> tuple[str, str | None, str | None]: ...

class SingleZipReader(zipfile.ZipFile):
    # Incompatible override
    #     zipfile.ZipFile.read(
    #         self,
    #         name: typing.Union[typing.Text, zipfile.ZipInfo],
    #         pwd: Optional[bytes] = ...,
    #     ) -> bytes: ...
    def read(self) -> bytes: ...  # type: ignore[override]

def humanize(dirname: str) -> str: ...



from typing import Any, ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

class PcxImageFile(ImageFile):
    format: ClassVar[Literal["PCX", "DCX"]]
    format_description: ClassVar[str]

SAVE: Any



from typing_extensions import TypeAlias

PY36: bool
PY37: bool
PY38: bool
PY39: bool
PY310: bool
PY311: bool

_VT: TypeAlias = tuple[int, int, int, str, int]

def get_version(version: _VT | None = ...) -> str: ...
def get_main_version(version: _VT = ...) -> str: ...
def get_complete_version(version: _VT | None = ...) -> _VT: ...
def get_docs_version(version: _VT | None = ...) -> str: ...
def get_git_changeset() -> str | None: ...
def get_version_tuple(version: str) -> tuple[int, int, int]: ...



import abc
import builtins
import codecs
import sys
from _typeshed import FileDescriptorOrPath, ReadableBuffer, WriteableBuffer
from collections.abc import Callable, Iterable, Iterator
from os import _Opener
from types import TracebackType
from typing import IO, Any, BinaryIO, Literal, Protocol, TextIO, TypeVar, overload, type_check_only
from typing_extensions import Self

__all__ = [
    "BlockingIOError",
    "open",
    "open_code",
    "IOBase",
    "RawIOBase",
    "FileIO",
    "BytesIO",
    "StringIO",
    "BufferedIOBase",
    "BufferedReader",
    "BufferedWriter",
    "BufferedRWPair",
    "BufferedRandom",
    "TextIOBase",
    "TextIOWrapper",
    "UnsupportedOperation",
    "SEEK_SET",
    "SEEK_CUR",
    "SEEK_END",
]

if sys.version_info >= (3, 11):
    __all__ += ["DEFAULT_BUFFER_SIZE", "IncrementalNewlineDecoder", "text_encoding"]

_T = TypeVar("_T")

DEFAULT_BUFFER_SIZE: Literal[8192]

SEEK_SET: Literal[0]
SEEK_CUR: Literal[1]
SEEK_END: Literal[2]

open = builtins.open

def open_code(path: str) -> IO[bytes]: ...

BlockingIOError = builtins.BlockingIOError

class UnsupportedOperation(OSError, ValueError): ...

class IOBase(metaclass=abc.ABCMeta):
    def __iter__(self) -> Iterator[bytes]: ...
    def __next__(self) -> bytes: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    def close(self) -> None: ...
    def fileno(self) -> int: ...
    def flush(self) -> None: ...
    def isatty(self) -> bool: ...
    def readable(self) -> bool: ...
    read: Callable[..., Any]
    def readlines(self, hint: int = -1, /) -> list[bytes]: ...
    def seek(self, offset: int, whence: int = ..., /) -> int: ...
    def seekable(self) -> bool: ...
    def tell(self) -> int: ...
    def truncate(self, size: int | None = ..., /) -> int: ...
    def writable(self) -> bool: ...
    write: Callable[..., Any]
    def writelines(self, lines: Iterable[ReadableBuffer], /) -> None: ...
    def readline(self, size: int | None = -1, /) -> bytes: ...
    def __del__(self) -> None: ...
    @property
    def closed(self) -> bool: ...
    def _checkClosed(self, msg: str | None = ...) -> None: ...  # undocumented

class RawIOBase(IOBase):
    def readall(self) -> bytes: ...
    def readinto(self, buffer: WriteableBuffer, /) -> int | None: ...
    def write(self, b: ReadableBuffer, /) -> int | None: ...
    def read(self, size: int = -1, /) -> bytes | None: ...

class BufferedIOBase(IOBase):
    raw: RawIOBase  # This is not part of the BufferedIOBase API and may not exist on some implementations.
    def detach(self) -> RawIOBase: ...
    def readinto(self, buffer: WriteableBuffer, /) -> int: ...
    def write(self, buffer: ReadableBuffer, /) -> int: ...
    def readinto1(self, buffer: WriteableBuffer, /) -> int: ...
    def read(self, size: int | None = ..., /) -> bytes: ...
    def read1(self, size: int = ..., /) -> bytes: ...

class FileIO(RawIOBase, BinaryIO):  # type: ignore[misc]  # incompatible definitions of writelines in the base classes
    mode: str
    # The type of "name" equals the argument passed in to the constructor,
    # but that can make FileIO incompatible with other I/O types that assume
    # "name" is a str. In the future, making FileIO generic might help.
    name: Any
    def __init__(
        self, file: FileDescriptorOrPath, mode: str = ..., closefd: bool = ..., opener: _Opener | None = ...
    ) -> None: ...
    @property
    def closefd(self) -> bool: ...
    def write(self, b: ReadableBuffer, /) -> int: ...
    def read(self, size: int = -1, /) -> bytes: ...
    def __enter__(self) -> Self: ...

class BytesIO(BufferedIOBase, BinaryIO):  # type: ignore[misc]  # incompatible definitions of methods in the base classes
    def __init__(self, initial_bytes: ReadableBuffer = ...) -> None: ...
    # BytesIO does not contain a "name" field. This workaround is necessary
    # to allow BytesIO sub-classes to add this field, as it is defined
    # as a read-only property on IO[].
    name: Any
    def __enter__(self) -> Self: ...
    def getvalue(self) -> bytes: ...
    def getbuffer(self) -> memoryview: ...
    def read1(self, size: int | None = -1, /) -> bytes: ...

class BufferedReader(BufferedIOBase, BinaryIO):  # type: ignore[misc]  # incompatible definitions of methods in the base classes
    def __enter__(self) -> Self: ...
    def __init__(self, raw: RawIOBase, buffer_size: int = ...) -> None: ...
    def peek(self, size: int = 0, /) -> bytes: ...

class BufferedWriter(BufferedIOBase, BinaryIO):  # type: ignore[misc]  # incompatible definitions of writelines in the base classes
    def __enter__(self) -> Self: ...
    def __init__(self, raw: RawIOBase, buffer_size: int = ...) -> None: ...
    def write(self, buffer: ReadableBuffer, /) -> int: ...

class BufferedRandom(BufferedReader, BufferedWriter):  # type: ignore[misc]  # incompatible definitions of methods in the base classes
    def __enter__(self) -> Self: ...
    def seek(self, target: int, whence: int = 0, /) -> int: ...  # stubtest needs this

class BufferedRWPair(BufferedIOBase):
    def __init__(self, reader: RawIOBase, writer: RawIOBase, buffer_size: int = ...) -> None: ...
    def peek(self, size: int = ..., /) -> bytes: ...

class TextIOBase(IOBase):
    encoding: str
    errors: str | None
    newlines: str | tuple[str, ...] | None
    def __iter__(self) -> Iterator[str]: ...  # type: ignore[override]
    def __next__(self) -> str: ...  # type: ignore[override]
    def detach(self) -> BinaryIO: ...
    def write(self, s: str, /) -> int: ...
    def writelines(self, lines: Iterable[str], /) -> None: ...  # type: ignore[override]
    def readline(self, size: int = ..., /) -> str: ...  # type: ignore[override]
    def readlines(self, hint: int = -1, /) -> list[str]: ...  # type: ignore[override]
    def read(self, size: int | None = ..., /) -> str: ...

@type_check_only
class _WrappedBuffer(Protocol):
    # "name" is wrapped by TextIOWrapper. Its type is inconsistent between
    # the various I/O types, see the comments on TextIOWrapper.name and
    # TextIO.name.
    @property
    def name(self) -> Any: ...
    @property
    def closed(self) -> bool: ...
    def read(self, size: int = ..., /) -> ReadableBuffer: ...
    # Optional: def read1(self, size: int, /) -> ReadableBuffer: ...
    def write(self, b: bytes, /) -> object: ...
    def flush(self) -> object: ...
    def close(self) -> object: ...
    def seekable(self) -> bool: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def truncate(self, size: int, /) -> int: ...
    def fileno(self) -> int: ...
    def isatty(self) -> int: ...
    # Optional: Only needs to be present if seekable() returns True.
    # def seek(self, offset: Literal[0], whence: Literal[2]) -> int: ...
    # def tell(self) -> int: ...

# TODO: Should be generic over the buffer type, but needs to wait for
# TypeVar defaults.
class TextIOWrapper(TextIOBase, TextIO):  # type: ignore[misc]  # incompatible definitions of write in the base classes
    def __init__(
        self,
        buffer: _WrappedBuffer,
        encoding: str | None = None,
        errors: str | None = None,
        newline: str | None = None,
        line_buffering: bool = False,
        write_through: bool = False,
    ) -> None: ...
    # Equals the "buffer" argument passed in to the constructor.
    @property
    def buffer(self) -> BinaryIO: ...
    @property
    def closed(self) -> bool: ...
    @property
    def line_buffering(self) -> bool: ...
    @property
    def write_through(self) -> bool: ...
    def reconfigure(
        self,
        *,
        encoding: str | None = None,
        errors: str | None = None,
        newline: str | None = None,
        line_buffering: bool | None = None,
        write_through: bool | None = None,
    ) -> None: ...
    # These are inherited from TextIOBase, but must exist in the stub to satisfy mypy.
    def __enter__(self) -> Self: ...
    def __iter__(self) -> Iterator[str]: ...  # type: ignore[override]
    def __next__(self) -> str: ...  # type: ignore[override]
    def writelines(self, lines: Iterable[str], /) -> None: ...  # type: ignore[override]
    def readline(self, size: int = -1, /) -> str: ...  # type: ignore[override]
    def readlines(self, hint: int = -1, /) -> list[str]: ...  # type: ignore[override]
    # Equals the "buffer" argument passed in to the constructor.
    def detach(self) -> BinaryIO: ...
    # TextIOWrapper's version of seek only supports a limited subset of
    # operations.
    def seek(self, cookie: int, whence: int = 0, /) -> int: ...

class StringIO(TextIOWrapper):
    def __init__(self, initial_value: str | None = ..., newline: str | None = ...) -> None: ...
    # StringIO does not contain a "name" field. This workaround is necessary
    # to allow StringIO sub-classes to add this field, as it is defined
    # as a read-only property on IO[].
    name: Any
    def getvalue(self) -> str: ...

class IncrementalNewlineDecoder(codecs.IncrementalDecoder):
    def __init__(self, decoder: codecs.IncrementalDecoder | None, translate: bool, errors: str = ...) -> None: ...
    def decode(self, input: ReadableBuffer | str, final: bool = False) -> str: ...
    @property
    def newlines(self) -> str | tuple[str, ...] | None: ...
    def setstate(self, state: tuple[bytes, int], /) -> None: ...

if sys.version_info >= (3, 10):
    @overload
    def text_encoding(encoding: None, stacklevel: int = 2, /) -> Literal["locale", "utf-8"]: ...
    @overload
    def text_encoding(encoding: _T, stacklevel: int = 2, /) -> _T: ...



from collections import UserList
from collections.abc import Mapping, Sequence
from datetime import datetime
from typing import Any

from django.core.exceptions import ValidationError
from django.core.files.uploadedfile import UploadedFile
from django.utils.datastructures import MultiValueDict
from django.utils.safestring import SafeString
from typing_extensions import TypeAlias

_DataT: TypeAlias = Mapping[str, Any]  # noqa: Y047

_FilesT: TypeAlias = MultiValueDict[str, UploadedFile]  # noqa: Y047

def pretty_name(name: str) -> str: ...
def flatatt(attrs: dict[str, Any]) -> SafeString: ...

class ErrorDict(dict):
    def as_data(self) -> dict[str, list[ValidationError]]: ...
    def get_json_data(self, escape_html: bool = ...) -> dict[str, Any]: ...
    def as_json(self, escape_html: bool = ...) -> str: ...
    def as_ul(self) -> str: ...
    def as_text(self) -> str: ...

class ErrorList(UserList):
    data: list[ValidationError | str]
    error_class: str
    def __init__(
        self,
        initlist: ErrorList | Sequence[str | Exception] | None = ...,
        error_class: str | None = ...,
    ) -> None: ...
    def as_data(self) -> list[ValidationError]: ...
    def get_json_data(self, escape_html: bool = ...) -> list[dict[str, str]]: ...
    def as_json(self, escape_html: bool = ...) -> str: ...
    def as_ul(self) -> str: ...
    def as_text(self) -> str: ...

def from_current_timezone(value: datetime) -> datetime: ...
def to_current_timezone(value: datetime) -> datetime: ...



from collections.abc import Iterable, Mapping
from typing import Any

from django.db.models.fields import Field
from django.db.models.lookups import (
    Exact,
    GreaterThan,
    GreaterThanOrEqual,
    In,
    IsNull,
    LessThan,
    LessThanOrEqual,
    Lookup,
)

class MultiColSource:
    alias: str
    field: Field
    sources: tuple[Field, Field]
    targets: tuple[Field, Field]
    contains_aggregate: bool
    output_field: Field
    def __init__(
        self, alias: str, targets: tuple[Field, Field], sources: tuple[Field, Field], field: Field
    ) -> None: ...
    def relabeled_clone(self, relabels: Mapping[str, str]) -> MultiColSource: ...
    def get_lookup(self, lookup: str) -> type[Lookup] | None: ...

def get_normalized_value(value: Any, lhs: Any) -> tuple[Any, ...]: ...

class RelatedIn(In):
    bilateral_transforms: list[Any]
    lhs: Any
    rhs: Any
    def get_prep_lookup(self) -> Iterable[Any]: ...

class RelatedLookupMixin:
    rhs: Any
    def get_prep_lookup(self) -> Any: ...

class RelatedExact(RelatedLookupMixin, Exact): ...
class RelatedLessThan(RelatedLookupMixin, LessThan): ...
class RelatedGreaterThan(RelatedLookupMixin, GreaterThan): ...
class RelatedGreaterThanOrEqual(RelatedLookupMixin, GreaterThanOrEqual): ...
class RelatedLessThanOrEqual(RelatedLookupMixin, LessThanOrEqual): ...
class RelatedIsNull(RelatedLookupMixin, IsNull): ...



from logging import Logger
from typing import Any

from django.forms.widgets import Widget

logger: Logger

class BaseGeometryWidget(Widget):
    geom_type: str
    map_srid: int
    map_width: int
    map_height: int
    display_raw: bool
    supports_3d: bool
    template_name: str
    attrs: Any
    def __init__(self, attrs: Any | None = ...) -> None: ...
    def serialize(self, value: Any) -> Any: ...
    def deserialize(self, value: Any) -> Any: ...
    def get_context(self, name: Any, value: Any, attrs: Any) -> Any: ...

class OpenLayersWidget(BaseGeometryWidget):
    template_name: str
    map_srid: int

    class Media:
        css: Any
        js: Any
    def serialize(self, value: Any) -> Any: ...
    def deserialize(self, value: Any) -> Any: ...

class OSMWidget(OpenLayersWidget):
    template_name: str
    default_lon: int
    default_lat: int
    default_zoom: int
    def __init__(self, attrs: Any | None = ...) -> None: ...



from typing_extensions import TypeAlias

# Actually Tuple[(int,) * 625]
_State: TypeAlias = tuple[int, ...]

class Random:
    def __init__(self, seed: object = ...) -> None: ...
    def seed(self, n: object = None, /) -> None: ...
    def getstate(self) -> _State: ...
    def setstate(self, state: _State, /) -> None: ...
    def random(self) -> float: ...
    def getrandbits(self, k: int, /) -> int: ...



from _typeshed import Incomplete
from typing import Any

modules: Any

def check_module(feature): ...
def version_module(feature): ...
def get_supported_modules(): ...

codecs: Any

def check_codec(feature): ...
def version_codec(feature): ...
def get_supported_codecs(): ...

features: Any

def check_feature(feature): ...
def version_feature(feature): ...
def get_supported_features(): ...
def check(feature): ...
def version(feature): ...
def get_supported(): ...
def pilinfo(out: Incomplete | None = None, supported_formats: bool = True) -> None: ...



import sys
from _typeshed import StrEnum, StrOrBytesPath
from collections.abc import Iterable
from cProfile import Profile as _cProfile
from profile import Profile
from typing import IO, Any, Literal, overload
from typing_extensions import Self, TypeAlias

if sys.version_info >= (3, 9):
    __all__ = ["Stats", "SortKey", "FunctionProfile", "StatsProfile"]
else:
    __all__ = ["Stats", "SortKey"]

_Selector: TypeAlias = str | float | int

class SortKey(StrEnum):
    CALLS: str
    CUMULATIVE: str
    FILENAME: str
    LINE: str
    NAME: str
    NFL: str
    PCALLS: str
    STDNAME: str
    TIME: str

if sys.version_info >= (3, 9):
    from dataclasses import dataclass

    @dataclass(unsafe_hash=True)
    class FunctionProfile:
        ncalls: str
        tottime: float
        percall_tottime: float
        cumtime: float
        percall_cumtime: float
        file_name: str
        line_number: int

    @dataclass(unsafe_hash=True)
    class StatsProfile:
        total_tt: float
        func_profiles: dict[str, FunctionProfile]

_SortArgDict: TypeAlias = dict[str, tuple[tuple[tuple[int, int], ...], str]]

class Stats:
    sort_arg_dict_default: _SortArgDict
    def __init__(
        self,
        arg: None | str | Profile | _cProfile = ...,
        /,
        *args: None | str | Profile | _cProfile | Self,
        stream: IO[Any] | None = None,
    ) -> None: ...
    def init(self, arg: None | str | Profile | _cProfile) -> None: ...
    def load_stats(self, arg: None | str | Profile | _cProfile) -> None: ...
    def get_top_level_stats(self) -> None: ...
    def add(self, *arg_list: None | str | Profile | _cProfile | Self) -> Self: ...
    def dump_stats(self, filename: StrOrBytesPath) -> None: ...
    def get_sort_arg_defs(self) -> _SortArgDict: ...
    @overload
    def sort_stats(self, field: Literal[-1, 0, 1, 2]) -> Self: ...
    @overload
    def sort_stats(self, *field: str) -> Self: ...
    def reverse_order(self) -> Self: ...
    def strip_dirs(self) -> Self: ...
    def calc_callees(self) -> None: ...
    def eval_print_amount(self, sel: _Selector, list: list[str], msg: str) -> tuple[list[str], str]: ...
    if sys.version_info >= (3, 9):
        def get_stats_profile(self) -> StatsProfile: ...

    def get_print_list(self, sel_list: Iterable[_Selector]) -> tuple[int, list[str]]: ...
    def print_stats(self, *amount: _Selector) -> Self: ...
    def print_callees(self, *amount: _Selector) -> Self: ...
    def print_callers(self, *amount: _Selector) -> Self: ...
    def print_call_heading(self, name_size: int, column_title: str) -> None: ...
    def print_call_line(self, name_size: int, source: str, call_dict: dict[str, Any], arrow: str = "->") -> None: ...
    def print_title(self) -> None: ...
    def print_line(self, func: str) -> None: ...



import datetime
import enum
import sys
from _typeshed import Unused
from collections.abc import Iterable, Sequence
from time import struct_time
from typing import ClassVar, Literal
from typing_extensions import TypeAlias

__all__ = [
    "IllegalMonthError",
    "IllegalWeekdayError",
    "setfirstweekday",
    "firstweekday",
    "isleap",
    "leapdays",
    "weekday",
    "monthrange",
    "monthcalendar",
    "prmonth",
    "month",
    "prcal",
    "calendar",
    "timegm",
    "month_name",
    "month_abbr",
    "day_name",
    "day_abbr",
    "Calendar",
    "TextCalendar",
    "HTMLCalendar",
    "LocaleTextCalendar",
    "LocaleHTMLCalendar",
    "weekheader",
]

if sys.version_info >= (3, 10):
    __all__ += ["FRIDAY", "MONDAY", "SATURDAY", "SUNDAY", "THURSDAY", "TUESDAY", "WEDNESDAY"]
if sys.version_info >= (3, 12):
    __all__ += [
        "Day",
        "Month",
        "JANUARY",
        "FEBRUARY",
        "MARCH",
        "APRIL",
        "MAY",
        "JUNE",
        "JULY",
        "AUGUST",
        "SEPTEMBER",
        "OCTOBER",
        "NOVEMBER",
        "DECEMBER",
    ]

_LocaleType: TypeAlias = tuple[str | None, str | None]

class IllegalMonthError(ValueError):
    def __init__(self, month: int) -> None: ...

class IllegalWeekdayError(ValueError):
    def __init__(self, weekday: int) -> None: ...

def isleap(year: int) -> bool: ...
def leapdays(y1: int, y2: int) -> int: ...
def weekday(year: int, month: int, day: int) -> int: ...
def monthrange(year: int, month: int) -> tuple[int, int]: ...

class Calendar:
    firstweekday: int
    def __init__(self, firstweekday: int = 0) -> None: ...
    def getfirstweekday(self) -> int: ...
    def setfirstweekday(self, firstweekday: int) -> None: ...
    def iterweekdays(self) -> Iterable[int]: ...
    def itermonthdates(self, year: int, month: int) -> Iterable[datetime.date]: ...
    def itermonthdays2(self, year: int, month: int) -> Iterable[tuple[int, int]]: ...
    def itermonthdays(self, year: int, month: int) -> Iterable[int]: ...
    def monthdatescalendar(self, year: int, month: int) -> list[list[datetime.date]]: ...
    def monthdays2calendar(self, year: int, month: int) -> list[list[tuple[int, int]]]: ...
    def monthdayscalendar(self, year: int, month: int) -> list[list[int]]: ...
    def yeardatescalendar(self, year: int, width: int = 3) -> list[list[int]]: ...
    def yeardays2calendar(self, year: int, width: int = 3) -> list[list[tuple[int, int]]]: ...
    def yeardayscalendar(self, year: int, width: int = 3) -> list[list[int]]: ...
    def itermonthdays3(self, year: int, month: int) -> Iterable[tuple[int, int, int]]: ...
    def itermonthdays4(self, year: int, month: int) -> Iterable[tuple[int, int, int, int]]: ...

class TextCalendar(Calendar):
    def prweek(self, theweek: int, width: int) -> None: ...
    def formatday(self, day: int, weekday: int, width: int) -> str: ...
    def formatweek(self, theweek: int, width: int) -> str: ...
    def formatweekday(self, day: int, width: int) -> str: ...
    def formatweekheader(self, width: int) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, width: int, withyear: bool = True) -> str: ...
    def prmonth(self, theyear: int, themonth: int, w: int = 0, l: int = 0) -> None: ...
    def formatmonth(self, theyear: int, themonth: int, w: int = 0, l: int = 0) -> str: ...
    def formatyear(self, theyear: int, w: int = 2, l: int = 1, c: int = 6, m: int = 3) -> str: ...
    def pryear(self, theyear: int, w: int = 0, l: int = 0, c: int = 6, m: int = 3) -> None: ...

def firstweekday() -> int: ...
def monthcalendar(year: int, month: int) -> list[list[int]]: ...
def prweek(theweek: int, width: int) -> None: ...
def week(theweek: int, width: int) -> str: ...
def weekheader(width: int) -> str: ...
def prmonth(theyear: int, themonth: int, w: int = 0, l: int = 0) -> None: ...
def month(theyear: int, themonth: int, w: int = 0, l: int = 0) -> str: ...
def calendar(theyear: int, w: int = 2, l: int = 1, c: int = 6, m: int = 3) -> str: ...
def prcal(theyear: int, w: int = 0, l: int = 0, c: int = 6, m: int = 3) -> None: ...

class HTMLCalendar(Calendar):
    cssclasses: ClassVar[list[str]]
    cssclass_noday: ClassVar[str]
    cssclasses_weekday_head: ClassVar[list[str]]
    cssclass_month_head: ClassVar[str]
    cssclass_month: ClassVar[str]
    cssclass_year: ClassVar[str]
    cssclass_year_head: ClassVar[str]
    def formatday(self, day: int, weekday: int) -> str: ...
    def formatweek(self, theweek: int) -> str: ...
    def formatweekday(self, day: int) -> str: ...
    def formatweekheader(self) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, withyear: bool = True) -> str: ...
    def formatmonth(self, theyear: int, themonth: int, withyear: bool = True) -> str: ...
    def formatyear(self, theyear: int, width: int = 3) -> str: ...
    def formatyearpage(
        self, theyear: int, width: int = 3, css: str | None = "calendar.css", encoding: str | None = None
    ) -> bytes: ...

class different_locale:
    def __init__(self, locale: _LocaleType) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, *args: Unused) -> None: ...

class LocaleTextCalendar(TextCalendar):
    def __init__(self, firstweekday: int = 0, locale: _LocaleType | None = None) -> None: ...

class LocaleHTMLCalendar(HTMLCalendar):
    def __init__(self, firstweekday: int = 0, locale: _LocaleType | None = None) -> None: ...
    def formatweekday(self, day: int) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, withyear: bool = True) -> str: ...

c: TextCalendar

def setfirstweekday(firstweekday: int) -> None: ...
def format(cols: int, colwidth: int = 20, spacing: int = 6) -> str: ...
def formatstring(cols: int, colwidth: int = 20, spacing: int = 6) -> str: ...
def timegm(tuple: tuple[int, ...] | struct_time) -> int: ...

# Data attributes
day_name: Sequence[str]
day_abbr: Sequence[str]
month_name: Sequence[str]
month_abbr: Sequence[str]

if sys.version_info >= (3, 12):
    class Month(enum.IntEnum):
        JANUARY: Literal[1]
        FEBRUARY: Literal[2]
        MARCH: Literal[3]
        APRIL: Literal[4]
        MAY: Literal[5]
        JUNE: Literal[6]
        JULY: Literal[7]
        AUGUST: Literal[8]
        SEPTEMBER: Literal[9]
        OCTOBER: Literal[10]
        NOVEMBER: Literal[11]
        DECEMBER: Literal[12]

    JANUARY = Month.JANUARY
    FEBRUARY = Month.FEBRUARY
    MARCH = Month.MARCH
    APRIL = Month.APRIL
    MAY = Month.MAY
    JUNE = Month.JUNE
    JULY = Month.JULY
    AUGUST = Month.AUGUST
    SEPTEMBER = Month.SEPTEMBER
    OCTOBER = Month.OCTOBER
    NOVEMBER = Month.NOVEMBER
    DECEMBER = Month.DECEMBER

    class Day(enum.IntEnum):
        MONDAY: Literal[0]
        TUESDAY: Literal[1]
        WEDNESDAY: Literal[2]
        THURSDAY: Literal[3]
        FRIDAY: Literal[4]
        SATURDAY: Literal[5]
        SUNDAY: Literal[6]

    MONDAY = Day.MONDAY
    TUESDAY = Day.TUESDAY
    WEDNESDAY = Day.WEDNESDAY
    THURSDAY = Day.THURSDAY
    FRIDAY = Day.FRIDAY
    SATURDAY = Day.SATURDAY
    SUNDAY = Day.SUNDAY
else:
    MONDAY: Literal[0]
    TUESDAY: Literal[1]
    WEDNESDAY: Literal[2]
    THURSDAY: Literal[3]
    FRIDAY: Literal[4]
    SATURDAY: Literal[5]
    SUNDAY: Literal[6]

EPOCH: Literal[1970]



from collections.abc import Callable
from typing import Any

def get_func_args(func: Callable[..., Any]) -> list[str]: ...
def get_func_full_args(func: Callable[..., Any]) -> list[tuple[str, str] | tuple[str]]: ...
def func_accepts_kwargs(func: Callable[..., Any]) -> bool: ...
def func_accepts_var_args(func: Callable[..., Any]) -> bool: ...
def method_has_no_args(meth: Callable[..., Any]) -> bool: ...
def func_supports_parameter(func: Callable[..., Any], name: str) -> bool: ...



from typing import List

from django.db.models.signals import ModelSignal

from nwon_django_toolbox.signals.disable_signals import DisableSignals

DISABLE_SIGNALS = DisableSignals([])


def disable_signals(signals: List[ModelSignal]):
    for signal in signals:
        DISABLE_SIGNALS.disconnect(signal)


def enable_signals(signals: List[ModelSignal]):
    for signal in signals:
        DISABLE_SIGNALS.reconnect(signal)



from collections.abc import Callable, Mapping, Sequence
from typing import Any

from typing_extensions import TypedDict

class DefaultsSettings(TypedDict, total=False):
    DEFAULT_RENDERER_CLASSES: Sequence[str]
    DEFAULT_PARSER_CLASSES: Sequence[str]
    DEFAULT_AUTHENTICATION_CLASSES: Sequence[str]
    DEFAULT_PERMISSION_CLASSES: Sequence[str]
    DEFAULT_THROTTLE_CLASSES: Sequence[str]
    DEFAULT_CONTENT_NEGOTIATION_CLASS: str
    DEFAULT_METADATA_CLASS: str
    DEFAULT_VERSIONING_CLASS: str | None
    DEFAULT_PAGINATION_CLASS: str | None
    DEFAULT_FILTER_BACKENDS: Sequence[str]
    DEFAULT_SCHEMA_CLASS: str
    DEFAULT_THROTTLE_RATES: dict[str, float | int | None]
    NUM_PROXIES: int | None
    PAGE_SIZE: int | None
    SEARCH_PARAM: str
    ORDERING_PARAM: str
    DEFAULT_VERSION: str | None
    ALLOWED_VERSIONS: str | None
    VERSION_PARAM: str
    UNAUTHENTICATED_USER: str
    UNAUTHENTICATED_TOKEN: str | None
    VIEW_NAME_FUNCTION: str
    VIEW_DESCRIPTION_FUNCTION: str
    EXCEPTION_HANDLER: str | Callable[[Any, Any], Any]
    NON_FIELD_ERRORS_KEY: str
    TEST_REQUEST_RENDERER_CLASSES: Sequence[str]
    TEST_REQUEST_DEFAULT_FORMAT: str
    URL_FORMAT_OVERRIDE: str
    FORMAT_SUFFIX_KWARG: str
    URL_FIELD_NAME: str
    DATE_FORMAT: str
    DATE_INPUT_FORMATS: Sequence[str]
    DATETIME_FORMAT: str
    DATETIME_INPUT_FORMATS: Sequence[str]
    TIME_FORMAT: str
    TIME_INPUT_FORMATS: Sequence[str]
    UNICODE_JSON: bool
    COMPACT_JSON: bool
    STRICT_JSON: bool
    COERCE_DECIMAL_TO_STRING: bool
    UPLOADED_FILES_USE_URL: bool
    HTML_SELECT_CUTOFF: int
    HTML_SELECT_CUTOFF_TEXT: str
    SCHEMA_COERCE_PATH_PK: bool
    SCHEMA_COERCE_METHOD_NAMES: dict[str, str]

DEFAULTS: DefaultsSettings
IMPORT_STRINGS: Sequence[str]
REMOVED_SETTINGS: Sequence[str]

def perform_import(val: Any | None, setting_name: str) -> Any | None: ...
def import_from_string(val: Any | None, setting_name: str) -> Any: ...

class APISettings:
    defaults: DefaultsSettings
    import_strings: Sequence[str]
    def __init__(
        self,
        user_settings: DefaultsSettings | None = ...,
        defaults: DefaultsSettings | None = ...,
        import_strings: Sequence[str] | None = ...,
    ): ...
    @property
    def user_settings(self) -> Mapping[str, Any]: ...
    def __getattr__(self, attr: str) -> Any: ...
    def __check_user_settings(self, user_settings: Mapping[str, Any]) -> Mapping[str, Any]: ...
    def reload(self) -> None: ...

class _Settings(APISettings):
    DEFAULT_RENDERER_CLASSES: Sequence[str]
    DEFAULT_PARSER_CLASSES: Sequence[str]
    DEFAULT_AUTHENTICATION_CLASSES: Sequence[str]
    DEFAULT_PERMISSION_CLASSES: Sequence[str]
    DEFAULT_THROTTLE_CLASSES: Sequence[str]
    DEFAULT_CONTENT_NEGOTIATION_CLASS: str
    DEFAULT_METADATA_CLASS: str
    DEFAULT_VERSIONING_CLASS: str | None
    DEFAULT_PAGINATION_CLASS: str | None
    DEFAULT_FILTER_BACKENDS: Sequence[str]
    DEFAULT_SCHEMA_CLASS: str
    DEFAULT_THROTTLE_RATES: dict[str, float | int | None]
    NUM_PROXIES: int | None
    PAGE_SIZE: int | None
    SEARCH_PARAM: str
    ORDERING_PARAM: str
    DEFAULT_VERSION: str | None
    ALLOWED_VERSIONS: str | None
    VERSION_PARAM: str
    UNAUTHENTICATED_USER: str
    UNAUTHENTICATED_TOKEN: str | None
    VIEW_NAME_FUNCTION: str
    VIEW_DESCRIPTION_FUNCTION: str
    EXCEPTION_HANDLER: str | Callable[[Any, Any], Any]
    NON_FIELD_ERRORS_KEY: str
    TEST_REQUEST_RENDERER_CLASSES: Sequence[str]
    TEST_REQUEST_DEFAULT_FORMAT: str
    URL_FORMAT_OVERRIDE: str
    FORMAT_SUFFIX_KWARG: str
    URL_FIELD_NAME: str
    DATE_FORMAT: str
    DATE_INPUT_FORMATS: Sequence[str]
    DATETIME_FORMAT: str
    DATETIME_INPUT_FORMATS: Sequence[str]
    TIME_FORMAT: str
    TIME_INPUT_FORMATS: Sequence[str]
    UNICODE_JSON: bool
    COMPACT_JSON: bool
    STRICT_JSON: bool
    COERCE_DECIMAL_TO_STRING: bool
    UPLOADED_FILES_USE_URL: bool
    HTML_SELECT_CUTOFF: int
    HTML_SELECT_CUTOFF_TEXT: str
    SCHEMA_COERCE_PATH_PK: bool
    SCHEMA_COERCE_METHOD_NAMES: dict[str, str]

api_settings: _Settings

def reload_api_settings(*args: Any, **kwargs: Any) -> None: ...



import sys
from _typeshed import ReadOnlyBuffer
from typing import Any, TypeVar, overload
from typing_extensions import Literal, TypeAlias, final

ucd_3_2_0: UCD
unidata_version: str

if sys.version_info < (3, 10):
    ucnhash_CAPI: Any

_T = TypeVar("_T")

def bidirectional(__chr: str) -> str: ...
def category(__chr: str) -> str: ...
def combining(__chr: str) -> int: ...
@overload
def decimal(__chr: str) -> int: ...
@overload
def decimal(__chr: str, __default: _T) -> int | _T: ...
def decomposition(__chr: str) -> str: ...
@overload
def digit(__chr: str) -> int: ...
@overload
def digit(__chr: str, __default: _T) -> int | _T: ...

_EastAsianWidth: TypeAlias = Literal["F", "H", "W", "Na", "A", "N"]

def east_asian_width(__chr: str) -> _EastAsianWidth: ...

if sys.version_info >= (3, 8):
    def is_normalized(__form: str, __unistr: str) -> bool: ...

def lookup(__name: str | ReadOnlyBuffer) -> str: ...
def mirrored(__chr: str) -> int: ...
@overload
def name(__chr: str) -> str: ...
@overload
def name(__chr: str, __default: _T) -> str | _T: ...
def normalize(__form: str, __unistr: str) -> str: ...
@overload
def numeric(__chr: str) -> float: ...
@overload
def numeric(__chr: str, __default: _T) -> float | _T: ...
@final
class UCD:
    # The methods below are constructed from the same array in C
    # (unicodedata_functions) and hence identical to the functions above.
    unidata_version: str
    def bidirectional(self, __chr: str) -> str: ...
    def category(self, __chr: str) -> str: ...
    def combining(self, __chr: str) -> int: ...
    @overload
    def decimal(self, __chr: str) -> int: ...
    @overload
    def decimal(self, __chr: str, __default: _T) -> int | _T: ...
    def decomposition(self, __chr: str) -> str: ...
    @overload
    def digit(self, __chr: str) -> int: ...
    @overload
    def digit(self, __chr: str, __default: _T) -> int | _T: ...
    def east_asian_width(self, __chr: str) -> _EastAsianWidth: ...
    if sys.version_info >= (3, 8):
        def is_normalized(self, __form: str, __unistr: str) -> bool: ...

    def lookup(self, __name: str | ReadOnlyBuffer) -> str: ...
    def mirrored(self, __chr: str) -> int: ...
    @overload
    def name(self, __chr: str) -> str: ...
    @overload
    def name(self, __chr: str, __default: _T) -> str | _T: ...
    def normalize(self, __form: str, __unistr: str) -> str: ...
    @overload
    def numeric(self, __chr: str) -> float: ...
    @overload
    def numeric(self, __chr: str, __default: _T) -> float | _T: ...



from _typeshed import StrPath, Unused
from collections.abc import Callable, Container, Iterable, Mapping
from typing import Any, Literal

def get_host_platform() -> str: ...
def get_platform() -> str: ...
def convert_path(pathname: str) -> str: ...
def change_root(new_root: str, pathname: str) -> str: ...
def check_environ() -> None: ...
def subst_vars(s: str, local_vars: Mapping[str, str]) -> None: ...
def split_quoted(s: str) -> list[str]: ...
def execute(
    func: Callable[..., object], args: tuple[Any, ...], msg: str | None = None, verbose: bool = ..., dry_run: bool = ...
) -> None: ...
def strtobool(val: str) -> Literal[0, 1]: ...
def byte_compile(
    py_files: list[str],
    optimize: int = 0,
    force: bool = ...,
    prefix: str | None = None,
    base_dir: str | None = None,
    verbose: bool = ...,
    dry_run: bool = ...,
    direct: bool | None = None,
) -> None: ...
def rfc822_escape(header: str) -> str: ...
def run_2to3(
    files: Iterable[str],
    fixer_names: Iterable[str] | None = None,
    options: Mapping[str, Any] | None = None,
    explicit: Unused = None,
) -> None: ...
def copydir_run_2to3(
    src: StrPath,
    dest: StrPath,
    template: str | None = None,
    fixer_names: Iterable[str] | None = None,
    options: Mapping[str, Any] | None = None,
    explicit: Container[str] | None = None,
) -> list[str]: ...

class Mixin2to3:
    fixer_names: Iterable[str] | None
    options: Mapping[str, Any] | None
    explicit: Container[str] | None
    def run_2to3(self, files: Iterable[str]) -> None: ...



from typing import Any

from ..cmd import Command

class clean(Command):
    description: str
    user_options: Any
    boolean_options: Any
    build_base: Any
    build_lib: Any
    build_temp: Any
    build_scripts: Any
    bdist_base: Any
    all: Any
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...



from typing import Any, List, Optional

from nwon_baseline.typings import AnyDict


def ensure_paged_results(
    api_result: AnyDict, expected_number_of_results: Optional[int] = None
) -> List[Any]:
    """
    Ensure a dictionary contains a results key with the expected number of results.
    """

    results = api_result["results"]

    assert isinstance(results, list), "'results' is not a list"

    if expected_number_of_results is not None:
        assert (
            len(results) == expected_number_of_results
        ), f"Expected {expected_number_of_results} result(s), but got {len(results)}"

    return results



import collections

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType


def prefetch_relations(queryset):
    """
    Based on:
    https://andrew.hawker.io/writings/2020/11/18/django-activity-stream-prefetch-generic-foreign-key/

    Prefetch content type relations for GenericForeignKeys to reduce N+1 queries.
    """
    # Get all GenericForeignKey fields on all models of the queryset.
    gfks = _queryset_generic_foreign_keys(queryset)

    # Get mapping of GFK content_type -> list of GFK object_pks for all GFK's on the queryset.
    gfks_data = _content_type_to_content_mapping_for_gfks(queryset, gfks)

    for content_type, object_pks in gfks_data.items():
        # Get all model instances referenced through a GFK.
        gfk_models = prefetch_relations(
            content_type.model_class()
            .objects.filter(pk__in=object_pks)
            .select_related()
        )

        for gfk_model in gfk_models:
            for gfk in _queryset_gfk_content_generator(queryset, gfks):
                qs_model, gfk_field_name, gfk_content_type, gfk_object_pk = gfk

                if gfk_content_type != content_type:
                    continue

                if gfk_object_pk != str(
                    gfk_model.pk
                ):  # str compare otherwise UUID PK's puke. :(
                    continue

                setattr(qs_model, gfk_field_name, gfk_model)

    return queryset


def _queryset_generic_foreign_keys(queryset):
    """
    Build mapping of name -> field for GenericForeignKey fields on the queryset.
    """

    gfks = {}
    for name, field in queryset.model.__dict__.items():
        if not isinstance(field, GenericForeignKey):
            continue

        gfks[name] = field

    return gfks


def _queryset_gfk_content_generator(queryset, gfks):
    """
    Generator function that yields information about all GenericForeignKey fields for all models of a queryset.
    """
    for model in queryset:
        for field_name, field in gfks.items():
            content_type_id = getattr(
                model, field.model._meta.get_field(field.ct_field).get_attname()
            )

            if not content_type_id:
                continue

            content_type = ContentType.objects.get_for_id(content_type_id)

            object_pk = str(getattr(model, field.fk_field))

            yield (model, field_name, content_type, object_pk)


def _content_type_to_content_mapping_for_gfks(queryset, gfks):
    """
    Build mapping of content_type -> [content_pk] for the given queryset and its generic foreign keys.
    """
    data = collections.defaultdict(list)

    for (
        model,
        field_name,
        content_type,
        object_pk,
    ) in _queryset_gfk_content_generator(queryset, gfks):
        data[content_type].append(object_pk)

    return data



from _typeshed import Incomplete
from enum import IntEnum
from typing import Any, ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

class LoadingStrategy(IntEnum):
    RGB_AFTER_FIRST = 0
    RGB_AFTER_DIFFERENT_PALETTE_ONLY = 1
    RGB_ALWAYS = 2

LOADING_STRATEGY: LoadingStrategy

class GifImageFile(ImageFile):
    format: ClassVar[Literal["GIF"]]
    format_description: ClassVar[str]
    global_palette: Any
    def data(self): ...
    @property
    def n_frames(self): ...
    @property
    def is_animated(self): ...
    im: Any
    def seek(self, frame) -> None: ...
    def tell(self): ...

RAWMODE: Any

def get_interlace(im): ...
def getheader(im, palette: Incomplete | None = None, info: Incomplete | None = None): ...
def getdata(im, offset=(0, 0), **params): ...



from collections.abc import Iterable, Sequence
from typing import Any

from django.db.backends.base.client import BaseDatabaseClient
from django.db.backends.mysql.base import DatabaseWrapper

class DatabaseClient(BaseDatabaseClient):
    connection: DatabaseWrapper
    executable_name: str
    @classmethod
    def settings_to_cmd_args_env(
        cls,
        settings_dict: dict[str, Any],
        parameters: Iterable[str],
    ) -> tuple[list[str], dict[str, str] | None]: ...



from collections.abc import Iterable, Mapping
from re import Pattern

ETAG_MATCH: Pattern[str]
MONTHS: list[str]
RFC1123_DATE: Pattern[str]
RFC850_DATE: Pattern[str]
ASCTIME_DATE: Pattern[str]
RFC3986_GENDELIMS: str
RFC3986_SUBDELIMS: str

def urlquote(url: str, safe: str = ...) -> str: ...
def urlquote_plus(url: str, safe: str = ...) -> str: ...
def urlunquote(quoted_url: str) -> str: ...
def urlunquote_plus(quoted_url: str) -> str: ...
def urlencode(
    query: Mapping[str, str | bytes | int | Iterable[str | bytes | int]]
    | Iterable[tuple[str, str | bytes | int | Iterable[str | bytes | int]]]
    | None,
    doseq: bool = ...,
) -> str: ...
def http_date(epoch_seconds: float | None = ...) -> str: ...
def parse_http_date(date: str) -> int: ...
def parse_http_date_safe(date: str) -> int | None: ...
def base36_to_int(s: str) -> int: ...
def int_to_base36(i: int) -> str: ...
def urlsafe_base64_encode(s: bytes) -> str: ...
def urlsafe_base64_decode(s: str) -> bytes: ...
def parse_etags(etag_str: str) -> list[str]: ...
def quote_etag(etag_str: str) -> str: ...
def is_same_domain(host: str, pattern: str) -> bool: ...
def url_has_allowed_host_and_scheme(
    url: str | None, allowed_hosts: str | Iterable[str] | None, require_https: bool = ...
) -> bool: ...
def is_safe_url(url: str | None, allowed_hosts: str | Iterable[str] | None, require_https: bool = ...) -> bool: ...
def parse_qsl(
    qs: str,
    keep_blank_values: bool = ...,
    strict_parsing: bool = ...,
    encoding: str = ...,
    errors: str = ...,
    max_num_fields: int | None = ...,
    separator: str = ...,
) -> list[tuple[str, str]]: ...
def escape_leading_slashes(url: str) -> str: ...



from collections.abc import Sequence
from typing import Any

from django.contrib.gis.gdal.driver import Driver
from django.contrib.gis.gdal.raster.band import BandList
from django.contrib.gis.gdal.raster.base import GDALRasterBase
from django.contrib.gis.gdal.srs import SpatialReference

class TransformPoint(list[Sequence[float]]):
    indices: dict[str, tuple[int, int]]
    def __init__(self, raster: GDALRaster, prop: str) -> None: ...
    @property
    def x(self) -> float: ...
    @x.setter
    def x(self, value: float) -> None: ...
    @property
    def y(self) -> float: ...
    @y.setter
    def y(self, value: float) -> None: ...

class GDALRaster(GDALRasterBase):
    destructor: Any
    def __init__(self, ds_input: Any, write: bool = ...) -> None: ...
    def __del__(self) -> None: ...
    @property
    def vsi_buffer(self) -> bytes | None: ...
    @property
    def is_vsi_based(self) -> bool: ...
    @property
    def name(self) -> str: ...
    @property
    def driver(self) -> Driver: ...
    @property
    def width(self) -> int: ...
    @property
    def height(self) -> int: ...
    @property
    def srs(self) -> SpatialReference | None: ...
    @srs.setter
    def srs(self, value: SpatialReference | int | str) -> None: ...
    @property
    def srid(self) -> int: ...
    @srid.setter
    def srid(self, value: int) -> None: ...
    @property
    def geotransform(self) -> list[float]: ...
    @geotransform.setter
    def geotransform(self, values: list[float]) -> None: ...
    @property
    def origin(self) -> TransformPoint: ...
    @property
    def scale(self) -> TransformPoint: ...
    @property
    def skew(self) -> TransformPoint: ...
    @property
    def extent(self) -> tuple[float, float, float, float]: ...
    @property
    def bands(self) -> BandList: ...
    def warp(self, ds_input: Any, resampling: str = ..., max_error: float = ...) -> GDALRaster: ...
    def transform(
        self,
        srs: SpatialReference | int | str,
        driver: Any | None = ...,
        name: str | None = ...,
        resampling: str = ...,
        max_error: float = ...,
    ) -> GDALRaster: ...
    @property
    def info(self) -> str: ...



from typing import Any

from django.core.management.base import BaseCommand

class Command(BaseCommand):
    verbosity: int
    def handle(self, *tablenames: list[str], **options: Any) -> None: ...
    def create_table(self, database: str, tablename: str, dry_run: bool) -> None: ...



from collections.abc import Iterable, Mapping
from typing import Any

import coreapi
import requests
import urllib3
from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import AnonymousUser
from django.http import HttpRequest
from django.test import testcases
from django.test.client import Client as DjangoClient
from django.test.client import ClientHandler
from django.test.client import RequestFactory as DjangoRequestFactory
from rest_framework.authtoken.models import Token
from rest_framework.request import Request
from rest_framework.response import _MonkeyPatchedResponse
from typing_extensions import TypeAlias

_GetDataType: TypeAlias = (
    Mapping[str, str | bytes | int | Iterable[str | bytes | int]]
    | Iterable[tuple[str, str | bytes | int | Iterable[str | bytes | int]]]
    | None
)

def force_authenticate(
    request: HttpRequest, user: AnonymousUser | AbstractBaseUser | None = ..., token: Token | None = ...
) -> None: ...

class HeaderDict(urllib3._collections.HTTPHeaderDict):
    def get_all(self, key: Any, default: Any): ...

class MockOriginalResponse:
    msg: Any
    closed: bool
    def __init__(self, headers: Any) -> None: ...
    def isclosed(self): ...
    def close(self) -> None: ...

class DjangoTestAdapter(requests.adapters.HTTPAdapter):
    app: Any
    factory: Any
    def __init__(self) -> None: ...
    def get_environ(self, request: Request): ...
    def send(self, request: Request, *args: Any, **kwargs: Any) -> requests.Response: ...  # type: ignore[override]
    def close(self) -> None: ...

class RequestsClient(requests.Session): ...

class CoreAPIClient(coreapi.Client):
    def __init__(self, *args: Any, **kwargs: Any): ...
    @property
    def session(self): ...

class APIRequestFactory(DjangoRequestFactory):
    renderer_classes_list: Any
    default_format: Any
    enforce_csrf_checks: Any
    renderer_classes: Any
    def __init__(self, enforce_csrf_checks: bool = ..., **defaults: Any) -> None: ...
    def request(self, **kwargs: Any) -> Request: ...  # type: ignore[override]
    def get(self, path: str, data: _GetDataType = ..., follow: bool = ..., **extra: Any) -> Request: ...  # type: ignore[override]
    def post(self, path: str, data: Any | None = ..., format: str | None = ..., content_type: str | None = ..., follow: bool = ..., **extra: Any) -> Request: ...  # type: ignore[override]
    def put(self, path: str, data: Any | None = ..., format: str | None = ..., content_type: str | None = ..., follow: bool = ..., **extra: Any) -> Request: ...  # type: ignore[override]
    def patch(self, path: str, data: Any | None = ..., format: str | None = ..., content_type: str | None = ..., follow: bool = ..., **extra: Any) -> Request: ...  # type: ignore[override]
    def delete(self, path: str, data: Any | None = ..., format: str | None = ..., content_type: str | None = ..., follow: bool = ..., **extra: Any) -> Request: ...  # type: ignore[override]
    def options(self, path: str, data: dict[str, str] | str = ..., format: str | None = ..., content_type: Any | None = ..., follow: bool = ..., **extra: Any) -> Request: ...  # type: ignore[override]
    def generic(  # type: ignore[override]
        self, method: str, path: str, data: str = ..., content_type: str = ..., secure: bool = ..., **extra: Any
    ) -> Request: ...

class ForceAuthClientHandler(ClientHandler):
    def __init__(self, *args: Any, **kwargs: Any): ...
    def get_response(self, request: Request) -> _MonkeyPatchedResponse: ...  # type: ignore[override]

class APIClient(APIRequestFactory, DjangoClient):
    def credentials(self, **kwargs: Any): ...
    def force_authenticate(
        self, user: AnonymousUser | AbstractBaseUser | None = ..., token: Token | None = ...
    ) -> None: ...
    def request(self, **kwargs: Any) -> _MonkeyPatchedResponse: ...  # type: ignore[override]
    def get(self, path: str, data: _GetDataType = ..., follow: bool = ..., **extra: Any) -> _MonkeyPatchedResponse: ...  # type: ignore[override]
    def post(self, path: str, data: Any | None = ..., format: str | None = ..., content_type: str | None = ..., follow: bool = ..., **extra: Any) -> _MonkeyPatchedResponse: ...  # type: ignore[override]
    def put(self, path: str, data: Any | None = ..., format: str | None = ..., content_type: str | None = ..., follow: bool = ..., **extra: Any) -> _MonkeyPatchedResponse: ...  # type: ignore[override]
    def patch(self, path: str, data: Any | None = ..., format: str | None = ..., content_type: str | None = ..., follow: bool = ..., **extra: Any) -> _MonkeyPatchedResponse: ...  # type: ignore[override]
    def delete(self, path: str, data: Any | None = ..., format: str | None = ..., content_type: str | None = ..., follow: bool = ..., **extra: Any) -> _MonkeyPatchedResponse: ...  # type: ignore[override]
    def options(self, path: str, data: dict[str, str] | str = ..., format: str | None = ..., content_type: Any | None = ..., follow: bool = ..., **extra: Any) -> _MonkeyPatchedResponse: ...  # type: ignore[override]
    def logout(self) -> None: ...

class APITransactionTestCase(testcases.TransactionTestCase):
    client_class: type[APIClient]
    client: APIClient

class APITestCase(testcases.TestCase):
    client_class: type[APIClient]
    client: APIClient

class APISimpleTestCase(testcases.SimpleTestCase):
    client_class: type[APIClient]
    client: APIClient

class APILiveServerTestCase(testcases.LiveServerTestCase):
    client_class: type[APIClient]
    client: APIClient

class URLPatternsTestCase(testcases.SimpleTestCase): ...



from typing import Any

from .backends.base.base import BaseDatabaseWrapper
from .utils import DEFAULT_DB_ALIAS as DEFAULT_DB_ALIAS  # Not exported in __all__
from .utils import DJANGO_VERSION_PICKLE_KEY as DJANGO_VERSION_PICKLE_KEY
from .utils import ConnectionHandler, ConnectionRouter
from .utils import DatabaseError as DatabaseError
from .utils import DataError as DataError
from .utils import Error as Error
from .utils import IntegrityError as IntegrityError
from .utils import InterfaceError as InterfaceError
from .utils import InternalError as InternalError
from .utils import NotSupportedError as NotSupportedError
from .utils import OperationalError as OperationalError
from .utils import ProgrammingError as ProgrammingError

connections: ConnectionHandler
router: ConnectionRouter
# Actually DefaultConnectionProxy, but quacks exactly like BaseDatabaseWrapper, it's not worth distinguishing the two.
connection: BaseDatabaseWrapper

def close_old_connections(**kwargs: Any) -> None: ...
def reset_queries(**kwargs: Any) -> None: ...



from typing import Any, AnyStr

from django.contrib.gis.gdal import CoordTransform, SpatialReference
from django.contrib.gis.gdal.base import GDALBase
from django.contrib.gis.gdal.envelope import Envelope
from django.contrib.gis.gdal.geomtype import OGRGeomType
from django.contrib.gis.geos import GEOSGeometry

class OGRGeometry(GDALBase):
    destructor: Any
    ptr: Any
    srs: SpatialReference | None
    def __init__(self, geom_input: Any, srs: SpatialReference | None = ...) -> None: ...
    @classmethod
    def from_bbox(cls, bbox: tuple[float, float, float, float]) -> OGRGeometry: ...
    @staticmethod
    def from_json(geom_input: AnyStr) -> OGRGeometry: ...
    @classmethod
    def from_gml(cls, gml_string: AnyStr) -> OGRGeometry: ...
    def __or__(self, other: OGRGeometry) -> OGRGeometry: ...
    def __and__(self, other: OGRGeometry) -> OGRGeometry: ...
    def __sub__(self, other: OGRGeometry) -> OGRGeometry: ...
    def __xor__(self, other: OGRGeometry) -> OGRGeometry: ...
    def __eq__(self, other: object) -> bool: ...
    @property
    def dimension(self) -> int: ...
    coord_dim: int
    @property
    def geom_count(self) -> int: ...
    @property
    def point_count(self) -> int: ...
    @property
    def num_points(self) -> int: ...
    @property
    def num_coords(self) -> int: ...
    @property
    def geom_type(self) -> OGRGeomType: ...
    @property
    def geom_name(self) -> str: ...
    @property
    def area(self) -> float: ...
    @property
    def envelope(self) -> Envelope: ...
    @property
    def empty(self) -> bool: ...
    @property
    def extent(self) -> tuple[float, float, float, float]: ...
    srid: int | None
    @property
    def geos(self) -> GEOSGeometry: ...
    @property
    def gml(self) -> str: ...
    @property
    def hex(self) -> bytes: ...
    @property
    def json(self) -> str: ...
    geojson: str
    @property
    def kml(self) -> str: ...
    @property
    def wkb_size(self) -> int: ...
    @property
    def wkb(self) -> memoryview: ...
    @property
    def wkt(self) -> str: ...
    @property
    def ewkt(self) -> str: ...
    def clone(self) -> Any: ...
    def close_rings(self) -> None: ...
    def transform(self, coord_trans: CoordTransform | SpatialReference | str | int, clone: bool = ...) -> Any: ...
    def intersects(self, other: OGRGeometry) -> bool: ...
    def equals(self, other: OGRGeometry) -> bool: ...
    def disjoint(self, other: OGRGeometry) -> bool: ...
    def touches(self, other: OGRGeometry) -> bool: ...
    def crosses(self, other: OGRGeometry) -> bool: ...
    def within(self, other: OGRGeometry) -> bool: ...
    def contains(self, other: OGRGeometry) -> bool: ...
    def overlaps(self, other: OGRGeometry) -> bool: ...
    @property
    def boundary(self) -> OGRGeometry: ...
    @property
    def convex_hull(self) -> OGRGeometry: ...
    def difference(self, other: OGRGeometry) -> OGRGeometry: ...
    def intersection(self, other: OGRGeometry) -> OGRGeometry: ...
    def sym_difference(self, other: OGRGeometry) -> OGRGeometry: ...
    def union(self, other: OGRGeometry) -> OGRGeometry: ...

class Point(OGRGeometry):
    @property
    def x(self) -> float: ...
    @property
    def y(self) -> float: ...
    @property
    def z(self) -> float | None: ...
    @property
    def coords(self) -> tuple[float, float] | tuple[float, float, float]: ...
    @property
    def tuple(self) -> tuple[float, float] | tuple[float, float, float]: ...

class LineString(OGRGeometry):
    def __getitem__(self, index: int) -> tuple[float, ...]: ...
    def __len__(self) -> int: ...
    @property
    def tuple(self) -> tuple[tuple[float, ...]]: ...
    coords: Any
    @property
    def x(self) -> list[float]: ...
    @property
    def y(self) -> list[float]: ...
    @property
    def z(self) -> list[float] | None: ...

class LinearRing(LineString): ...

class Polygon(OGRGeometry):
    def __len__(self) -> int: ...
    def __getitem__(self, index: int) -> OGRGeometry: ...
    @property
    def shell(self) -> OGRGeometry: ...
    exterior_ring: OGRGeometry
    @property
    def coords(self) -> tuple[tuple[tuple[float, ...]]]: ...
    @property
    def tuple(self) -> tuple[tuple[tuple[float, ...]]]: ...
    @property
    def point_count(self) -> int: ...
    @property
    def centroid(self) -> Point: ...

class GeometryCollection(OGRGeometry):
    def __getitem__(self, index: int) -> OGRGeometry: ...
    def __len__(self) -> int: ...
    def add(self, geom: OGRGeometry | str) -> None: ...
    @property
    def point_count(self) -> int: ...
    @property
    def coords(self) -> tuple[Any, ...]: ...
    @property
    def tuple(self) -> tuple[Any, ...]: ...

class MultiPoint(GeometryCollection): ...
class MultiLineString(GeometryCollection): ...
class MultiPolygon(GeometryCollection): ...

GEO_CLASSES: Any



from typing import BinaryIO

from nwon_baseline.file_helper import get_file_as_binary
from PIL import Image

from nwon_django_toolbox.tests.helper.random_temporary_file import (
    get_random_tempfile_path,
)


def get_random_image(width: int = 100, height: int = 100) -> BinaryIO:
    """
    Returns a randomly created image with the given dimensions as a binary
    """

    random_image_path = __path_to_created_random_image(width, height)
    binary = get_file_as_binary(random_image_path)

    if binary is None:
        raise Exception(f"Could not read image at {random_image_path}")

    return binary


def get_path_to_random_image(width: int = 100, height: int = 100) -> str:
    """
    Get a path to a randomly created image with the given dimensions.
    """

    return __path_to_created_random_image(width, height)


def __path_to_created_random_image(width: int, height: int) -> str:
    tempfile_path = get_random_tempfile_path(suffix="jpg")

    image = Image.new("RGB", size=(width, height))
    image.save(tempfile_path)

    return tempfile_path


__all__ = ["get_random_image", "get_path_to_random_image"]



from typing import Type

from django.db.models import Model
from rest_framework import status as rest_status
from rest_framework.response import Response
from rest_framework.serializers import Serializer

from nwon_django_toolbox.serializer.eager_loading_mixin import EagerLoadingMixin


def prefetched_response(
    model: Model, serializer: Type[Serializer], status=rest_status.HTTP_200_OK
):
    """
    A helper function that should be used to return a serialized instance
    in an API view.

    It basically ensures that eager loading (prefetching) is enforced before
    serializing the instance
    """

    query_set = model.__class__.objects.filter(pk=model.pk)

    if issubclass(serializer, EagerLoadingMixin):
        query_set = serializer.setup_eager_loading(query_set)

    return Response(
        serializer(query_set, many=True).data[0],
        status=status,
    )



from collections.abc import Mapping
from typing import IO, Any, Generic, TypeVar

from django.core.files.uploadedfile import UploadedFile
from django.http.request import _ImmutableQueryDict
from django.utils.datastructures import MultiValueDict
from rest_framework.renderers import JSONRenderer

_Data = TypeVar("_Data")
_Files = TypeVar("_Files")

class DataAndFiles(Generic[_Data, _Files]):
    data: _Data
    files: _Files
    def __init__(self, data: _Data, files: _Files) -> None: ...

class BaseParser:
    media_type: str
    def parse(
        self, stream: IO[Any], media_type: str | None = ..., parser_context: Mapping[str, Any] | None = ...
    ) -> Mapping[Any, Any] | DataAndFiles: ...

class JSONParser(BaseParser):
    renderer_class: type[JSONRenderer]
    strict: bool
    def parse(
        self, stream: IO[Any], media_type: str | None = ..., parser_context: Mapping[str, Any] | None = ...
    ) -> dict[str, Any]: ...

class FormParser(BaseParser):
    def parse(
        self, stream: IO[Any], media_type: str | None = ..., parser_context: Mapping[str, Any] | None = ...
    ) -> _ImmutableQueryDict: ...

class MultiPartParser(BaseParser):
    def parse(
        self, stream: IO[Any], media_type: str | None = ..., parser_context: Mapping[str, Any] | None = ...
    ) -> DataAndFiles[_ImmutableQueryDict, MultiValueDict]: ...

class FileUploadParser(BaseParser):
    errors: dict[str, str]
    def parse(
        self, stream: IO[Any], media_type: str | None = ..., parser_context: Mapping[str, Any] | None = ...
    ) -> DataAndFiles[None, Mapping[str, UploadedFile]]: ...
    def get_filename(self, stream: IO[Any], media_type: str | None, parser_context: Mapping[str, Any]) -> str: ...
    def get_encoded_filename(self, filename_parm: Mapping[str, Any]) -> str: ...



import sys
from _collections_abc import dict_items, dict_keys, dict_values
from _typeshed import SupportsItems, SupportsKeysAndGetItem, SupportsRichComparison, SupportsRichComparisonT
from typing import Any, Generic, NoReturn, SupportsIndex, TypeVar, final, overload
from typing_extensions import Self

if sys.version_info >= (3, 9):
    from types import GenericAlias

if sys.version_info >= (3, 10):
    from collections.abc import (
        Callable,
        ItemsView,
        Iterable,
        Iterator,
        KeysView,
        Mapping,
        MutableMapping,
        MutableSequence,
        Reversible,
        Sequence,
        ValuesView,
    )
else:
    from _collections_abc import *

__all__ = ["ChainMap", "Counter", "OrderedDict", "UserDict", "UserList", "UserString", "defaultdict", "deque", "namedtuple"]

_S = TypeVar("_S")
_T = TypeVar("_T")
_T1 = TypeVar("_T1")
_T2 = TypeVar("_T2")
_KT = TypeVar("_KT")
_VT = TypeVar("_VT")
_KT_co = TypeVar("_KT_co", covariant=True)
_VT_co = TypeVar("_VT_co", covariant=True)

# namedtuple is special-cased in the type checker; the initializer is ignored.
def namedtuple(
    typename: str,
    field_names: str | Iterable[str],
    *,
    rename: bool = False,
    module: str | None = None,
    defaults: Iterable[Any] | None = None,
) -> type[tuple[Any, ...]]: ...

class UserDict(MutableMapping[_KT, _VT]):
    data: dict[_KT, _VT]
    # __init__ should be kept roughly in line with `dict.__init__`, which has the same semantics
    @overload
    def __init__(self, dict: None = None, /) -> None: ...
    @overload
    def __init__(self: UserDict[str, _VT], dict: None = None, /, **kwargs: _VT) -> None: ...
    @overload
    def __init__(self, dict: SupportsKeysAndGetItem[_KT, _VT], /) -> None: ...
    @overload
    def __init__(self: UserDict[str, _VT], dict: SupportsKeysAndGetItem[str, _VT], /, **kwargs: _VT) -> None: ...
    @overload
    def __init__(self, iterable: Iterable[tuple[_KT, _VT]], /) -> None: ...
    @overload
    def __init__(self: UserDict[str, _VT], iterable: Iterable[tuple[str, _VT]], /, **kwargs: _VT) -> None: ...
    @overload
    def __init__(self: UserDict[str, str], iterable: Iterable[list[str]], /) -> None: ...
    @overload
    def __init__(self: UserDict[bytes, bytes], iterable: Iterable[list[bytes]], /) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: _KT) -> _VT: ...
    def __setitem__(self, key: _KT, item: _VT) -> None: ...
    def __delitem__(self, key: _KT) -> None: ...
    def __iter__(self) -> Iterator[_KT]: ...
    def __contains__(self, key: object) -> bool: ...
    def copy(self) -> Self: ...
    def __copy__(self) -> Self: ...

    # `UserDict.fromkeys` has the same semantics as `dict.fromkeys`, so should be kept in line with `dict.fromkeys`.
    # TODO: Much like `dict.fromkeys`, the true signature of `UserDict.fromkeys` is inexpressible in the current type system.
    # See #3800 & https://github.com/python/typing/issues/548#issuecomment-683336963.
    @classmethod
    @overload
    def fromkeys(cls, iterable: Iterable[_T], value: None = None) -> UserDict[_T, Any | None]: ...
    @classmethod
    @overload
    def fromkeys(cls, iterable: Iterable[_T], value: _S) -> UserDict[_T, _S]: ...
    if sys.version_info >= (3, 9):
        @overload
        def __or__(self, other: UserDict[_KT, _VT] | dict[_KT, _VT]) -> Self: ...
        @overload
        def __or__(self, other: UserDict[_T1, _T2] | dict[_T1, _T2]) -> UserDict[_KT | _T1, _VT | _T2]: ...
        @overload
        def __ror__(self, other: UserDict[_KT, _VT] | dict[_KT, _VT]) -> Self: ...
        @overload
        def __ror__(self, other: UserDict[_T1, _T2] | dict[_T1, _T2]) -> UserDict[_KT | _T1, _VT | _T2]: ...
        # UserDict.__ior__ should be kept roughly in line with MutableMapping.update()
        @overload  # type: ignore[misc]
        def __ior__(self, other: SupportsKeysAndGetItem[_KT, _VT]) -> Self: ...
        @overload
        def __ior__(self, other: Iterable[tuple[_KT, _VT]]) -> Self: ...
    if sys.version_info >= (3, 12):
        @overload
        def get(self, key: _KT, default: None = None) -> _VT | None: ...
        @overload
        def get(self, key: _KT, default: _T) -> _VT | _T: ...

class UserList(MutableSequence[_T]):
    data: list[_T]
    @overload
    def __init__(self, initlist: None = None) -> None: ...
    @overload
    def __init__(self, initlist: Iterable[_T]) -> None: ...
    def __lt__(self, other: list[_T] | UserList[_T]) -> bool: ...
    def __le__(self, other: list[_T] | UserList[_T]) -> bool: ...
    def __gt__(self, other: list[_T] | UserList[_T]) -> bool: ...
    def __ge__(self, other: list[_T] | UserList[_T]) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __contains__(self, item: object) -> bool: ...
    def __len__(self) -> int: ...
    @overload
    def __getitem__(self, i: SupportsIndex) -> _T: ...
    @overload
    def __getitem__(self, i: slice) -> Self: ...
    @overload
    def __setitem__(self, i: SupportsIndex, item: _T) -> None: ...
    @overload
    def __setitem__(self, i: slice, item: Iterable[_T]) -> None: ...
    def __delitem__(self, i: SupportsIndex | slice) -> None: ...
    def __add__(self, other: Iterable[_T]) -> Self: ...
    def __radd__(self, other: Iterable[_T]) -> Self: ...
    def __iadd__(self, other: Iterable[_T]) -> Self: ...
    def __mul__(self, n: int) -> Self: ...
    def __rmul__(self, n: int) -> Self: ...
    def __imul__(self, n: int) -> Self: ...
    def append(self, item: _T) -> None: ...
    def insert(self, i: int, item: _T) -> None: ...
    def pop(self, i: int = -1) -> _T: ...
    def remove(self, item: _T) -> None: ...
    def copy(self) -> Self: ...
    def __copy__(self) -> Self: ...
    def count(self, item: _T) -> int: ...
    # The runtime signature is "item, *args", and the arguments are then passed
    # to `list.index`. In order to give more precise types, we pretend that the
    # `item` argument is positional-only.
    def index(self, item: _T, start: SupportsIndex = 0, stop: SupportsIndex = sys.maxsize, /) -> int: ...
    # All arguments are passed to `list.sort` at runtime, so the signature should be kept in line with `list.sort`.
    @overload
    def sort(self: UserList[SupportsRichComparisonT], *, key: None = None, reverse: bool = False) -> None: ...
    @overload
    def sort(self, *, key: Callable[[_T], SupportsRichComparison], reverse: bool = False) -> None: ...
    def extend(self, other: Iterable[_T]) -> None: ...

class UserString(Sequence[UserString]):
    data: str
    def __init__(self, seq: object) -> None: ...
    def __int__(self) -> int: ...
    def __float__(self) -> float: ...
    def __complex__(self) -> complex: ...
    def __getnewargs__(self) -> tuple[str]: ...
    def __lt__(self, string: str | UserString) -> bool: ...
    def __le__(self, string: str | UserString) -> bool: ...
    def __gt__(self, string: str | UserString) -> bool: ...
    def __ge__(self, string: str | UserString) -> bool: ...
    def __eq__(self, string: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __contains__(self, char: object) -> bool: ...
    def __len__(self) -> int: ...
    def __getitem__(self, index: SupportsIndex | slice) -> Self: ...
    def __iter__(self) -> Iterator[Self]: ...
    def __reversed__(self) -> Iterator[Self]: ...
    def __add__(self, other: object) -> Self: ...
    def __radd__(self, other: object) -> Self: ...
    def __mul__(self, n: int) -> Self: ...
    def __rmul__(self, n: int) -> Self: ...
    def __mod__(self, args: Any) -> Self: ...
    def __rmod__(self, template: object) -> Self: ...
    def capitalize(self) -> Self: ...
    def casefold(self) -> Self: ...
    def center(self, width: int, *args: Any) -> Self: ...
    def count(self, sub: str | UserString, start: int = 0, end: int = sys.maxsize) -> int: ...
    def encode(self: UserString, encoding: str | None = "utf-8", errors: str | None = "strict") -> bytes: ...
    def endswith(self, suffix: str | tuple[str, ...], start: int | None = 0, end: int | None = sys.maxsize) -> bool: ...
    def expandtabs(self, tabsize: int = 8) -> Self: ...
    def find(self, sub: str | UserString, start: int = 0, end: int = sys.maxsize) -> int: ...
    def format(self, *args: Any, **kwds: Any) -> str: ...
    def format_map(self, mapping: Mapping[str, Any]) -> str: ...
    def index(self, sub: str, start: int = 0, end: int = sys.maxsize) -> int: ...
    def isalpha(self) -> bool: ...
    def isalnum(self) -> bool: ...
    def isdecimal(self) -> bool: ...
    def isdigit(self) -> bool: ...
    def isidentifier(self) -> bool: ...
    def islower(self) -> bool: ...
    def isnumeric(self) -> bool: ...
    def isprintable(self) -> bool: ...
    def isspace(self) -> bool: ...
    def istitle(self) -> bool: ...
    def isupper(self) -> bool: ...
    def isascii(self) -> bool: ...
    def join(self, seq: Iterable[str]) -> str: ...
    def ljust(self, width: int, *args: Any) -> Self: ...
    def lower(self) -> Self: ...
    def lstrip(self, chars: str | None = None) -> Self: ...
    maketrans = str.maketrans
    def partition(self, sep: str) -> tuple[str, str, str]: ...
    if sys.version_info >= (3, 9):
        def removeprefix(self, prefix: str | UserString, /) -> Self: ...
        def removesuffix(self, suffix: str | UserString, /) -> Self: ...

    def replace(self, old: str | UserString, new: str | UserString, maxsplit: int = -1) -> Self: ...
    def rfind(self, sub: str | UserString, start: int = 0, end: int = sys.maxsize) -> int: ...
    def rindex(self, sub: str | UserString, start: int = 0, end: int = sys.maxsize) -> int: ...
    def rjust(self, width: int, *args: Any) -> Self: ...
    def rpartition(self, sep: str) -> tuple[str, str, str]: ...
    def rstrip(self, chars: str | None = None) -> Self: ...
    def split(self, sep: str | None = None, maxsplit: int = -1) -> list[str]: ...
    def rsplit(self, sep: str | None = None, maxsplit: int = -1) -> list[str]: ...
    def splitlines(self, keepends: bool = False) -> list[str]: ...
    def startswith(self, prefix: str | tuple[str, ...], start: int | None = 0, end: int | None = sys.maxsize) -> bool: ...
    def strip(self, chars: str | None = None) -> Self: ...
    def swapcase(self) -> Self: ...
    def title(self) -> Self: ...
    def translate(self, *args: Any) -> Self: ...
    def upper(self) -> Self: ...
    def zfill(self, width: int) -> Self: ...

class deque(MutableSequence[_T]):
    @property
    def maxlen(self) -> int | None: ...
    @overload
    def __init__(self, *, maxlen: int | None = None) -> None: ...
    @overload
    def __init__(self, iterable: Iterable[_T], maxlen: int | None = None) -> None: ...
    def append(self, x: _T, /) -> None: ...
    def appendleft(self, x: _T, /) -> None: ...
    def copy(self) -> Self: ...
    def count(self, x: _T, /) -> int: ...
    def extend(self, iterable: Iterable[_T], /) -> None: ...
    def extendleft(self, iterable: Iterable[_T], /) -> None: ...
    def insert(self, i: int, x: _T, /) -> None: ...
    def index(self, x: _T, start: int = 0, stop: int = ..., /) -> int: ...
    def pop(self) -> _T: ...  # type: ignore[override]
    def popleft(self) -> _T: ...
    def remove(self, value: _T, /) -> None: ...
    def rotate(self, n: int = 1, /) -> None: ...
    def __copy__(self) -> Self: ...
    def __len__(self) -> int: ...
    # These methods of deque don't take slices, unlike MutableSequence, hence the type: ignores
    def __getitem__(self, key: SupportsIndex, /) -> _T: ...  # type: ignore[override]
    def __setitem__(self, key: SupportsIndex, value: _T, /) -> None: ...  # type: ignore[override]
    def __delitem__(self, key: SupportsIndex, /) -> None: ...  # type: ignore[override]
    def __contains__(self, key: object, /) -> bool: ...
    def __reduce__(self) -> tuple[type[Self], tuple[()], None, Iterator[_T]]: ...
    def __iadd__(self, value: Iterable[_T], /) -> Self: ...
    def __add__(self, value: Self, /) -> Self: ...
    def __mul__(self, value: int, /) -> Self: ...
    def __imul__(self, value: int, /) -> Self: ...
    def __lt__(self, value: deque[_T], /) -> bool: ...
    def __le__(self, value: deque[_T], /) -> bool: ...
    def __gt__(self, value: deque[_T], /) -> bool: ...
    def __ge__(self, value: deque[_T], /) -> bool: ...
    def __eq__(self, value: object, /) -> bool: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any, /) -> GenericAlias: ...

class Counter(dict[_T, int], Generic[_T]):
    @overload
    def __init__(self, iterable: None = None, /) -> None: ...
    @overload
    def __init__(self: Counter[str], iterable: None = None, /, **kwargs: int) -> None: ...
    @overload
    def __init__(self, mapping: SupportsKeysAndGetItem[_T, int], /) -> None: ...
    @overload
    def __init__(self, iterable: Iterable[_T], /) -> None: ...
    def copy(self) -> Self: ...
    def elements(self) -> Iterator[_T]: ...
    def most_common(self, n: int | None = None) -> list[tuple[_T, int]]: ...
    @classmethod
    def fromkeys(cls, iterable: Any, v: int | None = None) -> NoReturn: ...  # type: ignore[override]
    @overload
    def subtract(self, iterable: None = None, /) -> None: ...
    @overload
    def subtract(self, mapping: Mapping[_T, int], /) -> None: ...
    @overload
    def subtract(self, iterable: Iterable[_T], /) -> None: ...
    # Unlike dict.update(), use Mapping instead of SupportsKeysAndGetItem for the first overload
    # (source code does an `isinstance(other, Mapping)` check)
    #
    # The second overload is also deliberately different to dict.update()
    # (if it were `Iterable[_T] | Iterable[tuple[_T, int]]`,
    # the tuples would be added as keys, breaking type safety)
    @overload  # type: ignore[override]
    def update(self, m: Mapping[_T, int], /, **kwargs: int) -> None: ...
    @overload
    def update(self, iterable: Iterable[_T], /, **kwargs: int) -> None: ...
    @overload
    def update(self, iterable: None = None, /, **kwargs: int) -> None: ...
    def __missing__(self, key: _T) -> int: ...
    def __delitem__(self, elem: object) -> None: ...
    if sys.version_info >= (3, 10):
        def __eq__(self, other: object) -> bool: ...
        def __ne__(self, other: object) -> bool: ...

    def __add__(self, other: Counter[_S]) -> Counter[_T | _S]: ...
    def __sub__(self, other: Counter[_T]) -> Counter[_T]: ...
    def __and__(self, other: Counter[_T]) -> Counter[_T]: ...
    def __or__(self, other: Counter[_S]) -> Counter[_T | _S]: ...  # type: ignore[override]
    def __pos__(self) -> Counter[_T]: ...
    def __neg__(self) -> Counter[_T]: ...
    # several type: ignores because __iadd__ is supposedly incompatible with __add__, etc.
    def __iadd__(self, other: SupportsItems[_T, int]) -> Self: ...  # type: ignore[misc]
    def __isub__(self, other: SupportsItems[_T, int]) -> Self: ...
    def __iand__(self, other: SupportsItems[_T, int]) -> Self: ...
    def __ior__(self, other: SupportsItems[_T, int]) -> Self: ...  # type: ignore[override,misc]
    if sys.version_info >= (3, 10):
        def total(self) -> int: ...
        def __le__(self, other: Counter[Any]) -> bool: ...
        def __lt__(self, other: Counter[Any]) -> bool: ...
        def __ge__(self, other: Counter[Any]) -> bool: ...
        def __gt__(self, other: Counter[Any]) -> bool: ...

# The pure-Python implementations of the "views" classes
# These are exposed at runtime in `collections/__init__.py`
class _OrderedDictKeysView(KeysView[_KT_co], Reversible[_KT_co]):
    def __reversed__(self) -> Iterator[_KT_co]: ...

class _OrderedDictItemsView(ItemsView[_KT_co, _VT_co], Reversible[tuple[_KT_co, _VT_co]]):
    def __reversed__(self) -> Iterator[tuple[_KT_co, _VT_co]]: ...

class _OrderedDictValuesView(ValuesView[_VT_co], Reversible[_VT_co]):
    def __reversed__(self) -> Iterator[_VT_co]: ...

# The C implementations of the "views" classes
# (At runtime, these are called `odict_keys`, `odict_items` and `odict_values`,
# but they are not exposed anywhere)
# pyright doesn't have a specific error code for subclassing error!
@final
class _odict_keys(dict_keys[_KT_co, _VT_co], Reversible[_KT_co]):  # type: ignore[misc]  # pyright: ignore
    def __reversed__(self) -> Iterator[_KT_co]: ...

@final
class _odict_items(dict_items[_KT_co, _VT_co], Reversible[tuple[_KT_co, _VT_co]]):  # type: ignore[misc]  # pyright: ignore
    def __reversed__(self) -> Iterator[tuple[_KT_co, _VT_co]]: ...

@final
class _odict_values(dict_values[_KT_co, _VT_co], Reversible[_VT_co], Generic[_KT_co, _VT_co]):  # type: ignore[misc]  # pyright: ignore
    def __reversed__(self) -> Iterator[_VT_co]: ...

class OrderedDict(dict[_KT, _VT], Reversible[_KT], Generic[_KT, _VT]):
    def popitem(self, last: bool = True) -> tuple[_KT, _VT]: ...
    def move_to_end(self, key: _KT, last: bool = True) -> None: ...
    def copy(self) -> Self: ...
    def __reversed__(self) -> Iterator[_KT]: ...
    def keys(self) -> _odict_keys[_KT, _VT]: ...
    def items(self) -> _odict_items[_KT, _VT]: ...
    def values(self) -> _odict_values[_KT, _VT]: ...
    # The signature of OrderedDict.fromkeys should be kept in line with `dict.fromkeys`, modulo positional-only differences.
    # Like dict.fromkeys, its true signature is not expressible in the current type system.
    # See #3800 & https://github.com/python/typing/issues/548#issuecomment-683336963.
    @classmethod
    @overload
    def fromkeys(cls, iterable: Iterable[_T], value: None = None) -> OrderedDict[_T, Any | None]: ...
    @classmethod
    @overload
    def fromkeys(cls, iterable: Iterable[_T], value: _S) -> OrderedDict[_T, _S]: ...
    # Keep OrderedDict.setdefault in line with MutableMapping.setdefault, modulo positional-only differences.
    @overload
    def setdefault(self: OrderedDict[_KT, _T | None], key: _KT, default: None = None) -> _T | None: ...
    @overload
    def setdefault(self, key: _KT, default: _VT) -> _VT: ...
    # Same as dict.pop, but accepts keyword arguments
    @overload
    def pop(self, key: _KT) -> _VT: ...
    @overload
    def pop(self, key: _KT, default: _VT) -> _VT: ...
    @overload
    def pop(self, key: _KT, default: _T) -> _VT | _T: ...
    def __eq__(self, value: object, /) -> bool: ...
    if sys.version_info >= (3, 9):
        @overload
        def __or__(self, value: dict[_KT, _VT], /) -> Self: ...
        @overload
        def __or__(self, value: dict[_T1, _T2], /) -> OrderedDict[_KT | _T1, _VT | _T2]: ...
        @overload
        def __ror__(self, value: dict[_KT, _VT], /) -> Self: ...
        @overload
        def __ror__(self, value: dict[_T1, _T2], /) -> OrderedDict[_KT | _T1, _VT | _T2]: ...  # type: ignore[misc]

class defaultdict(dict[_KT, _VT]):
    default_factory: Callable[[], _VT] | None
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self: defaultdict[str, _VT], **kwargs: _VT) -> None: ...
    @overload
    def __init__(self, default_factory: Callable[[], _VT] | None, /) -> None: ...
    @overload
    def __init__(self: defaultdict[str, _VT], default_factory: Callable[[], _VT] | None, /, **kwargs: _VT) -> None: ...
    @overload
    def __init__(self, default_factory: Callable[[], _VT] | None, map: SupportsKeysAndGetItem[_KT, _VT], /) -> None: ...
    @overload
    def __init__(
        self: defaultdict[str, _VT],
        default_factory: Callable[[], _VT] | None,
        map: SupportsKeysAndGetItem[str, _VT],
        /,
        **kwargs: _VT,
    ) -> None: ...
    @overload
    def __init__(self, default_factory: Callable[[], _VT] | None, iterable: Iterable[tuple[_KT, _VT]], /) -> None: ...
    @overload
    def __init__(
        self: defaultdict[str, _VT],
        default_factory: Callable[[], _VT] | None,
        iterable: Iterable[tuple[str, _VT]],
        /,
        **kwargs: _VT,
    ) -> None: ...
    def __missing__(self, key: _KT, /) -> _VT: ...
    def __copy__(self) -> Self: ...
    def copy(self) -> Self: ...
    if sys.version_info >= (3, 9):
        @overload
        def __or__(self, value: dict[_KT, _VT], /) -> Self: ...
        @overload
        def __or__(self, value: dict[_T1, _T2], /) -> defaultdict[_KT | _T1, _VT | _T2]: ...
        @overload
        def __ror__(self, value: dict[_KT, _VT], /) -> Self: ...
        @overload
        def __ror__(self, value: dict[_T1, _T2], /) -> defaultdict[_KT | _T1, _VT | _T2]: ...  # type: ignore[misc]

class ChainMap(MutableMapping[_KT, _VT]):
    maps: list[MutableMapping[_KT, _VT]]
    def __init__(self, *maps: MutableMapping[_KT, _VT]) -> None: ...
    def new_child(self, m: MutableMapping[_KT, _VT] | None = None) -> Self: ...
    @property
    def parents(self) -> Self: ...
    def __setitem__(self, key: _KT, value: _VT) -> None: ...
    def __delitem__(self, key: _KT) -> None: ...
    def __getitem__(self, key: _KT) -> _VT: ...
    def __iter__(self) -> Iterator[_KT]: ...
    def __len__(self) -> int: ...
    def __contains__(self, key: object) -> bool: ...
    @overload
    def get(self, key: _KT, default: None = None) -> _VT | None: ...
    @overload
    def get(self, key: _KT, default: _T) -> _VT | _T: ...
    def __missing__(self, key: _KT) -> _VT: ...  # undocumented
    def __bool__(self) -> bool: ...
    # Keep ChainMap.setdefault in line with MutableMapping.setdefault, modulo positional-only differences.
    @overload
    def setdefault(self: ChainMap[_KT, _T | None], key: _KT, default: None = None) -> _T | None: ...
    @overload
    def setdefault(self, key: _KT, default: _VT) -> _VT: ...
    @overload
    def pop(self, key: _KT) -> _VT: ...
    @overload
    def pop(self, key: _KT, default: _VT) -> _VT: ...
    @overload
    def pop(self, key: _KT, default: _T) -> _VT | _T: ...
    def copy(self) -> Self: ...
    __copy__ = copy
    # All arguments to `fromkeys` are passed to `dict.fromkeys` at runtime, so the signature should be kept in line with `dict.fromkeys`.
    @classmethod
    @overload
    def fromkeys(cls, iterable: Iterable[_T]) -> ChainMap[_T, Any | None]: ...
    @classmethod
    @overload
    # Special-case None: the user probably wants to add non-None values later.
    def fromkeys(cls, iterable: Iterable[_T], value: None, /) -> ChainMap[_T, Any | None]: ...
    @classmethod
    @overload
    def fromkeys(cls, iterable: Iterable[_T], value: _S, /) -> ChainMap[_T, _S]: ...
    if sys.version_info >= (3, 9):
        @overload
        def __or__(self, other: Mapping[_KT, _VT]) -> Self: ...
        @overload
        def __or__(self, other: Mapping[_T1, _T2]) -> ChainMap[_KT | _T1, _VT | _T2]: ...
        @overload
        def __ror__(self, other: Mapping[_KT, _VT]) -> Self: ...
        @overload
        def __ror__(self, other: Mapping[_T1, _T2]) -> ChainMap[_KT | _T1, _VT | _T2]: ...
        # ChainMap.__ior__ should be kept roughly in line with MutableMapping.update()
        @overload  # type: ignore[misc]
        def __ior__(self, other: SupportsKeysAndGetItem[_KT, _VT]) -> Self: ...
        @overload
        def __ior__(self, other: Iterable[tuple[_KT, _VT]]) -> Self: ...



from typing import IO, Any
from xml.sax.expatreader import ExpatParser as _ExpatParser

from django.core.serializers import base

class Serializer(base.Serializer):
    def indent(self, level: int) -> None: ...
    xml: Any
    def start_serialization(self) -> None: ...
    def end_serialization(self) -> None: ...
    def start_object(self, obj: Any) -> None: ...
    def end_object(self, obj: Any) -> None: ...
    def handle_field(self, obj: Any, field: Any) -> None: ...
    def handle_fk_field(self, obj: Any, field: Any) -> None: ...
    def handle_m2m_field(self, obj: Any, field: Any) -> None: ...

class Deserializer(base.Deserializer):
    handle_forward_references: bool
    event_stream: Any
    db: str
    ignore: bool
    def __init__(
        self,
        stream_or_string: bytes | str | IO[bytes] | IO[str],
        *,
        using: str = ...,
        ignorenonexistent: bool = ...,
        **options: Any
    ) -> None: ...
    def __next__(self) -> base.DeserializedObject: ...

def getInnerText(node: Any) -> str: ...

class DefusedExpatParser(_ExpatParser):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def start_doctype_decl(self, name: Any, sysid: Any, pubid: Any, has_internal_subset: Any) -> None: ...
    def entity_decl(
        self, name: Any, is_parameter_entity: Any, value: Any, base: Any, sysid: Any, pubid: Any, notation_name: Any
    ) -> None: ...
    def unparsed_entity_decl(self, name: Any, base: Any, sysid: Any, pubid: Any, notation_name: Any) -> None: ...
    def external_entity_ref_handler(self, context: Any, base: Any, sysid: Any, pubid: Any) -> None: ...
    def reset(self) -> None: ...

class DefusedXmlException(ValueError): ...

class DTDForbidden(DefusedXmlException):
    name: Any
    sysid: Any
    pubid: Any
    def __init__(self, name: Any, sysid: Any, pubid: Any) -> None: ...

class EntitiesForbidden(DefusedXmlException):
    name: Any
    value: Any
    base: Any
    sysid: Any
    pubid: Any
    notation_name: Any
    def __init__(self, name: Any, value: Any, base: Any, sysid: Any, pubid: Any, notation_name: Any) -> None: ...

class ExternalReferenceForbidden(DefusedXmlException):
    context: Any
    base: Any
    sysid: Any
    pubid: Any
    def __init__(self, context: Any, base: Any, sysid: Any, pubid: Any) -> None: ...



from django.db.backends.base.operations import BaseDatabaseOperations
from django.db.backends.postgresql.base import DatabaseWrapper

class DatabaseOperations(BaseDatabaseOperations):
    connection: DatabaseWrapper



import json
from typing import List, Optional, Tuple, Union

from nwon_baseline.directory_helper import (
    file_paths_in_directory,
    get_latest_file_in_directory,
)
from nwon_baseline.file_helper import read_file_content

from nwon_django_toolbox.tests.celery.celery_folders import get_path_to_celery_folder
from nwon_django_toolbox.typings.celery.celery_folders import CeleryFolder


def count_messages(celery_folder: CeleryFolder) -> int:
    path = get_path_to_celery_folder(celery_folder)

    return len(file_paths_in_directory(path)) if path is not None else 0


def path_to_latest_celery_message(celery_folder: CeleryFolder) -> Union[str, None]:
    path = get_path_to_celery_folder(celery_folder)
    return get_latest_file_in_directory(path) if path is not None else None


def celery_messages_information(celery_folder: CeleryFolder) -> List[str]:
    return [
        "task " + task + " in queue " + queue
        for (task, queue) in enqueued_tasks_and_messages(celery_folder)
    ]


def enqueued_tasks_and_messages(celery_folder: CeleryFolder) -> List[Tuple[str, str]]:
    """
    Returns a Tuple (task_name, queue_name)
    """

    path = get_path_to_celery_folder(celery_folder)
    files = file_paths_in_directory(path) if path else []

    return [(task_name_for_message(file), queue_for_message(file)) for file in files]


def enqueued_tasks(celery_folder: CeleryFolder) -> List[str]:
    path = get_path_to_celery_folder(celery_folder)
    files = file_paths_in_directory(path) if path else []

    return [task_name_for_message(file) for file in files]


def task_name_for_message(message_path: str) -> str:
    message = __read_celery_message(message_path)
    return message["headers"]["task"] if message else "unknown"


def queue_for_message(message_path: str) -> str:
    message = __read_celery_message(message_path)
    return (
        message["properties"]["delivery_info"]["routing_key"] if message else "unknown"
    )


def __read_celery_message(file_path: str) -> Optional[dict]:
    content = read_file_content(file_path)
    return json.loads(content) if content is not None else None



import sys
import threading
from typing_extensions import Never

_global_lock: threading.Lock

class _LoopBoundMixin:
    if sys.version_info < (3, 11):
        def __init__(self, *, loop: Never = ...) -> None: ...



from typing import Any, Generic, List, Optional, Type, TypeVar, Union

from django.core.exceptions import ValidationError
from django.db.models import JSONField
from nwon_baseline.pydantic import pydantic_model_to_dict
from nwon_baseline.typings import AnyDict
from pydantic import BaseModel
from pydantic import ValidationError as PydanticValidationError

PydanticModel = TypeVar("PydanticModel", bound=BaseModel)


class PydanticJsonField(JSONField, Generic[PydanticModel]):
    """
    An extension of a JSONField that uses a pydantic model for data validation.
    """

    pydantic_model: Optional[Type[PydanticModel]] = None

    def __init__(
        self, *args, pydantic_model: Optional[Type[PydanticModel]] = None, **kwargs
    ):
        self.pydantic_model = pydantic_model
        if self.pydantic_model is not None and not issubclass(
            self.pydantic_model, BaseModel
        ):
            raise TypeError(
                "Parameter pydantic_model must be set and must be valid Pydantic model"
            )

        super().__init__(*args, **kwargs)

    def __get__(self, instance, owner) -> Optional[PydanticModel]:  # type: ignore
        """Ensure correct type hinting when accessing the field on a model instance."""
        return super().__get__(instance, owner)  # type: ignore

    def validate(
        self, value: Optional[Union[BaseModel, AnyDict, None]], model_instance
    ):
        if value is None:
            super().validate(value, model_instance)
        else:
            if isinstance(value, dict):
                super().validate(value, model_instance)
            else:
                super().validate(pydantic_model_to_dict(value), model_instance)

        self._validate_schema(value)

    def from_db_value(self, value, expression, connection) -> Optional[PydanticModel]:
        """
        Directly returns the value. Problem when returning a
        Pydantic model (which is what we want) is that it is not JSON serializable.
        """

        return self._validate_schema(value) if value is not None else None

    def to_python(self, value: Any) -> Optional[AnyDict]:
        """
        Directly returns the value. Problem when returning a
        Pydantic model (which is what we want) is that it is not JSON serializable.
        """

        if isinstance(value, BaseModel):
            return pydantic_model_to_dict(value)

        return value

    def get_prep_value(self, value):
        """
        Before saving the value we make sure it is a dictionary.

        Has the advantage that you can pass in a Pydantic model and it will be
        converted to a dictionary automatically.
        """

        if isinstance(value, BaseModel):
            return pydantic_model_to_dict(value)
        if isinstance(value, dict):
            return value  # Already a dictionary

        raise ValueError(f"Cannot store {type(value)} in {self.name}")

    def _validate_schema(
        self, value: Union[str, AnyDict, BaseModel, None]
    ) -> Optional[PydanticModel]:
        """
        Validates the value against the provided Pydantic model.
        Return the Pydantic model instance or raises ValidationError.
        """

        # Skip validation during fake migrations
        if self.model.__module__ == "__fake__":
            return value  # type: ignore

        if value is None or self.pydantic_model is None:
            return None

        errors: List[str] = []

        try:
            if isinstance(value, str):
                return self.pydantic_model.model_validate_json(value)
            else:
                return self.pydantic_model.model_validate(value)

        except PydanticValidationError as exc:
            errors.append(
                f"JSON does not fit Pydantic model {self.pydantic_model.__name__} {format(exc)}"
            )

        # Raise a validation error if no model matches
        raise ValidationError(
            f"Value does not match any Pydantic model: {errors}", code="invalid"
        )



from nwon_django_toolbox.models.update_validation.allow_updates_for_model import (
    allow_updates_for_model,
)
from nwon_django_toolbox.models.update_validation.validate_model_update import (
    validate_model_update,
)
from nwon_django_toolbox.typings import AllowedUpdate

__all__ = ["allow_updates_for_model", "validate_model_update", "AllowedUpdate"]



def make_archive(
    base_name: str,
    format: str,
    root_dir: str | None = None,
    base_dir: str | None = None,
    verbose: int = 0,
    dry_run: int = 0,
    owner: str | None = None,
    group: str | None = None,
) -> str: ...
def make_tarball(
    base_name: str,
    base_dir: str,
    compress: str | None = "gzip",
    verbose: int = 0,
    dry_run: int = 0,
    owner: str | None = None,
    group: str | None = None,
) -> str: ...
def make_zipfile(base_name: str, base_dir: str, verbose: int = 0, dry_run: int = 0) -> str: ...



import sys
from _typeshed import StrOrLiteralStr
from collections.abc import Iterable, Mapping, Sequence
from re import Pattern, RegexFlag
from typing import Any, ClassVar, overload
from typing_extensions import LiteralString, TypeAlias

__all__ = [
    "ascii_letters",
    "ascii_lowercase",
    "ascii_uppercase",
    "capwords",
    "digits",
    "hexdigits",
    "octdigits",
    "printable",
    "punctuation",
    "whitespace",
    "Formatter",
    "Template",
]

ascii_letters: LiteralString
ascii_lowercase: LiteralString
ascii_uppercase: LiteralString
digits: LiteralString
hexdigits: LiteralString
octdigits: LiteralString
punctuation: LiteralString
printable: LiteralString
whitespace: LiteralString

def capwords(s: StrOrLiteralStr, sep: StrOrLiteralStr | None = None) -> StrOrLiteralStr: ...

if sys.version_info >= (3, 9):
    _TemplateMetaclass: TypeAlias = type
else:
    class _TemplateMetaclass(type):
        pattern: ClassVar[str]
        def __init__(cls, name: str, bases: tuple[type, ...], dct: dict[str, Any]) -> None: ...

class Template(metaclass=_TemplateMetaclass):
    template: str
    delimiter: ClassVar[str]
    idpattern: ClassVar[str]
    braceidpattern: ClassVar[str | None]
    flags: ClassVar[RegexFlag]
    pattern: ClassVar[Pattern[str]]
    def __init__(self, template: str) -> None: ...
    def substitute(self, __mapping: Mapping[str, object] = {}, **kwds: object) -> str: ...
    def safe_substitute(self, __mapping: Mapping[str, object] = {}, **kwds: object) -> str: ...
    if sys.version_info >= (3, 11):
        def get_identifiers(self) -> list[str]: ...
        def is_valid(self) -> bool: ...

class Formatter:
    @overload
    def format(self, __format_string: LiteralString, *args: LiteralString, **kwargs: LiteralString) -> LiteralString: ...
    @overload
    def format(self, __format_string: str, *args: Any, **kwargs: Any) -> str: ...
    @overload
    def vformat(
        self, format_string: LiteralString, args: Sequence[LiteralString], kwargs: Mapping[LiteralString, LiteralString]
    ) -> LiteralString: ...
    @overload
    def vformat(self, format_string: str, args: Sequence[Any], kwargs: Mapping[str, Any]) -> str: ...
    def _vformat(  # undocumented
        self,
        format_string: str,
        args: Sequence[Any],
        kwargs: Mapping[str, Any],
        used_args: set[int | str],
        recursion_depth: int,
        auto_arg_index: int = 0,
    ) -> tuple[str, int]: ...
    def parse(
        self, format_string: StrOrLiteralStr
    ) -> Iterable[tuple[StrOrLiteralStr, StrOrLiteralStr | None, StrOrLiteralStr | None, StrOrLiteralStr | None]]: ...
    def get_field(self, field_name: str, args: Sequence[Any], kwargs: Mapping[str, Any]) -> Any: ...
    def get_value(self, key: int | str, args: Sequence[Any], kwargs: Mapping[str, Any]) -> Any: ...
    def check_unused_args(self, used_args: set[int | str], args: Sequence[Any], kwargs: Mapping[str, Any]) -> None: ...
    def format_field(self, value: Any, format_spec: str) -> Any: ...
    def convert_field(self, value: Any, conversion: str) -> Any: ...



import json
from collections.abc import Iterator
from typing import IO, Any

from django.core.serializers.base import DeserializedObject
from django.core.serializers.python import Serializer as PythonSerializer

class Serializer(PythonSerializer):
    json_kwargs: dict[str, Any]

def Deserializer(
    stream_or_string: IO[bytes] | IO[str] | bytes | str, **options: Any
) -> Iterator[DeserializedObject]: ...

class DjangoJSONEncoder(json.JSONEncoder):
    allow_nan: bool
    check_circular: bool
    ensure_ascii: bool
    indent: int
    skipkeys: bool
    sort_keys: bool



from ssl import _PeerCertRetDictType
from typing_extensions import Final

__version__: Final[str]

class CertificateError(ValueError): ...

def match_hostname(cert: _PeerCertRetDictType, hostname: str) -> None: ...



from collections.abc import Callable, Container, Iterable, MutableMapping
from typing import Any, Protocol, TypeVar

from django.db.models import Model, QuerySet
from rest_framework.fields import Field
from rest_framework.serializers import BaseSerializer
from typing_extensions import TypeAlias

_T = TypeVar("_T", bound=Model)
_V = TypeVar("_V", contravariant=True)

class ContextValidator(Protocol[_V]):
    requires_context: bool
    def __call__(self, __value: _V, __context: Field) -> None: ...

Validator: TypeAlias = Callable[[_V], None] | ContextValidator[_V]

def qs_exists(queryset: QuerySet) -> bool: ...
def qs_filter(queryset: QuerySet[_T], **kwargs: Any) -> QuerySet[_T]: ...

class UniqueValidator:
    message: str
    requires_context: bool
    queryset: QuerySet
    lookup: str
    def __init__(self, queryset: QuerySet, message: str | None = ..., lookup: str = ...) -> None: ...
    def filter_queryset(self, value: Any, queryset: QuerySet[_T], field_name: str) -> QuerySet[_T]: ...
    def exclude_current_instance(self, queryset: QuerySet[_T], instance: _T) -> QuerySet[_T]: ...
    def __call__(self, value: Any, serializer_field: Field) -> None: ...

class UniqueTogetherValidator:
    message: str
    missing_message: str
    requires_context: bool
    queryset: QuerySet
    fields: Iterable[str]
    def __init__(self, queryset: QuerySet, fields: Iterable[str], message: str | None = ...) -> None: ...
    def enforce_required_fields(self, attrs: Container[str], serializer: BaseSerializer) -> None: ...
    def filter_queryset(
        self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T], serializer: BaseSerializer
    ) -> QuerySet[_T]: ...
    def exclude_current_instance(
        self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T], instance: _T
    ) -> QuerySet[_T]: ...
    def __call__(self, attrs: MutableMapping[str, Any], serializer: BaseSerializer) -> None: ...

class ProhibitSurrogateCharactersValidator:
    message: str
    code: str
    def __call__(self, value: Any) -> None: ...

class BaseUniqueForValidator:
    message: str
    missing_message: str
    requires_context: bool
    queryset: QuerySet
    field: str
    date_field: str
    def __init__(self, queryset: QuerySet, field: str, date_field: str, message: str | None = ...) -> None: ...
    def enforce_required_fields(self, attrs: Container[str]) -> None: ...
    def filter_queryset(
        self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T], field_name: str, date_field_name: str
    ) -> QuerySet[_T]: ...
    def exclude_current_instance(
        self, attrs: MutableMapping[str, Any], queryset: QuerySet[_T], instance: Model
    ) -> QuerySet[_T]: ...
    def __call__(self, attrs: MutableMapping[str, Any], serializer: BaseSerializer) -> None: ...

class UniqueForDateValidator(BaseUniqueForValidator): ...
class UniqueForMonthValidator(BaseUniqueForValidator): ...
class UniqueForYearValidator(BaseUniqueForValidator): ...



from collections.abc import Container, Iterator
from typing import Any

from django.db.backends.base.base import BaseDatabaseWrapper
from typing_extensions import Literal

from .client import DatabaseClient
from .creation import DatabaseCreation
from .features import DatabaseFeatures
from .introspection import DatabaseIntrospection
from .operations import DatabaseOperations
from .validation import DatabaseValidation

version: Any
django_conversions: Any
server_version_re: Any

class CursorWrapper:
    codes_for_integrityerror: Any
    cursor: Any
    def __init__(self, cursor: Any) -> None: ...
    def execute(self, query: Any, args: Any | None = ...) -> Any: ...
    def executemany(self, query: Any, args: Any) -> Any: ...
    def __getattr__(self, attr: Any) -> Any: ...
    def __iter__(self) -> Iterator[Any]: ...

class DatabaseWrapper(BaseDatabaseWrapper):
    client: DatabaseClient
    creation: DatabaseCreation
    features: DatabaseFeatures
    introspection: DatabaseIntrospection
    validation: DatabaseValidation
    ops: DatabaseOperations

    client_class: type[DatabaseClient]
    creation_class: type[DatabaseCreation]
    features_class: type[DatabaseFeatures]
    introspection_class: type[DatabaseIntrospection]
    ops_class: type[DatabaseOperations]
    validation_class: type[DatabaseValidation]

    vendor: str
    data_types: Any
    operators: Any
    pattern_esc: str
    pattern_ops: Any
    isolation_levels: Any
    Database: Any
    SchemaEditorClass: Any
    isolation_level: Any
    def get_connection_params(self) -> dict[str, Any]: ...
    def get_new_connection(self, conn_params: Any) -> Any: ...
    def init_connection_state(self) -> None: ...
    def create_cursor(self, name: Any | None = ...) -> CursorWrapper: ...
    def disable_constraint_checking(self) -> Literal[True]: ...
    needs_rollback: Any
    def enable_constraint_checking(self) -> None: ...
    def check_constraints(self, table_names: Any | None = ...) -> None: ...
    def is_usable(self) -> bool: ...
    @property
    def display_name(self) -> str: ...  # type: ignore [override]
    @property
    def data_type_check_constraints(self) -> dict[str, str]: ...  # type: ignore [override]
    @property
    def mysql_server_data(self) -> dict[str, Any]: ...
    @property
    def mysql_server_info(self) -> str: ...
    @property
    def mysql_version(self) -> tuple[int, ...]: ...
    @property
    def mysql_is_mariadb(self) -> bool: ...
    @property
    def sql_mode(self) -> Container[str]: ...



from typing import Union

from django.conf import settings
from django.db.models.fields.files import FieldFile, ImageFieldFile
from django.http import HttpResponseRedirect


def redirect_to_s3_url(field: Union[ImageFieldFile, FieldFile]) -> HttpResponseRedirect:
    url = field.url.replace(settings.AWS_S3_ENDPOINT_URL, settings.AWS_S3_CUSTOM_DOMAIN)
    return HttpResponseRedirect(url)



from datetime import date
from typing import Any

TIME_STRINGS: dict[str, str]
TIMESINCE_CHUNKS: Any

def timesince(
    d: date,
    now: date | None = ...,
    reversed: bool = ...,
    time_strings: dict[str, str] | None = ...,
    depth: int = ...,
) -> str: ...
def timeuntil(d: date, now: date | None = ..., time_strings: dict[str, str] | None = ..., depth: int = ...) -> str: ...



from typing import Any

def srs_double(f: Any) -> Any: ...
def units_func(f: Any) -> Any: ...

clone_srs: Any
new_srs: Any
release_srs: Any
destroy_srs: Any
srs_validate: Any
set_axis_strategy: Any
semi_major: Any
semi_minor: Any
invflattening: Any
from_wkt: Any
from_proj: Any
from_epsg: Any
from_xml: Any
from_user_input: Any
morph_to_esri: Any
morph_from_esri: Any
identify_epsg: Any
linear_units: Any
angular_units: Any
to_wkt: Any
to_proj: Any
to_pretty_wkt: Any
to_xml: Any
get_attr_value: Any
get_auth_name: Any
get_auth_code: Any
isgeographic: Any
islocal: Any
isprojected: Any
new_ct: Any
destroy_ct: Any



from collections.abc import Callable, Sequence
from typing import Any, NamedTuple, TypeVar

from coreapi import Field as CoreAPIField
from django.core.paginator import Page, Paginator
from django.db.models import Model, QuerySet
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.views import APIView
from typing_extensions import TypedDict

def _positive_int(integer_string: str, strict: bool = ..., cutoff: int | None = ...) -> int: ...
def _divide_with_ceil(a: int, b: int) -> int: ...
def _get_displayed_page_numbers(current: int, final: int) -> list[int | None]: ...
def _get_page_links(
    page_numbers: Sequence[int | None], current: int, url_func: Callable[[int], str]
) -> list[PageLink]: ...
def _reverse_ordering(ordering_tuple: Sequence[str]) -> tuple[str, ...]: ...

class Cursor(NamedTuple):
    offset: int
    reverse: bool
    position: int | None

class PageLink(NamedTuple):
    url: str | None
    number: int | None
    is_active: bool
    is_break: bool

class HtmlContext(TypedDict):
    previous_url: str
    next_url: str

class HtmlContextWithPageLinks(HtmlContext):
    page_links: list[PageLink]

PAGE_BREAK: PageLink

_MT = TypeVar("_MT", bound=Model)

class BasePagination:
    display_page_controls: bool
    def get_paginated_response_schema(self, schema: Any): ...
    def get_paginated_response(self, data: Any) -> Response: ...
    def get_results(self, data: dict[str, Any]) -> Any: ...
    def get_schema_fields(self, view: APIView) -> list: ...
    def get_schema_operation_parameters(self, view: APIView) -> list: ...
    def paginate_queryset(self, queryset: QuerySet, request: Request, view: APIView | None = ...): ...
    def to_html(self) -> str: ...

class PageNumberPagination(BasePagination):
    display_page_controls: bool
    django_paginator_class: type[Paginator]
    invalid_page_message: str
    last_page_strings: Sequence[str]
    max_page_size: int | None
    page_query_description: str
    page_query_param: str
    page_size_query_description: str
    page_size_query_param: str | None
    page_size: int | None
    page: Page | None
    request: Request | None
    template: str
    def paginate_queryset(
        self, queryset: QuerySet[_MT], request: Request, view: APIView | None = ...
    ) -> list[_MT] | None: ...
    def get_paginated_response_schema(self, schema: dict[str, Any]) -> dict[str, Any]: ...
    def get_schema_fields(self, view: APIView) -> list[CoreAPIField]: ...
    def get_schema_operation_parameters(self, view: APIView) -> list[dict[str, Any]]: ...
    def get_page_number(self, request: Request, paginator: Paginator) -> int | str: ...
    def get_page_size(self, request: Request) -> int | None: ...
    def get_next_link(self) -> str | None: ...
    def get_previous_link(self) -> str | None: ...
    def get_html_context(self) -> HtmlContextWithPageLinks: ...

class LimitOffsetPagination(BasePagination):
    count: int | None
    default_limit: int | None
    limit_query_description: str
    limit_query_param: str
    limit: int | None
    max_limit: int | None
    offset_query_description: str
    offset_query_param: str
    offset: int | None
    request: Request | None
    template: str
    def paginate_queryset(
        self, queryset: QuerySet[_MT], request: Request, view: APIView | None = ...
    ) -> list[_MT] | None: ...
    def get_limit(self, request: Request) -> int | None: ...
    def get_offset(self, request: Request) -> int: ...
    def get_next_link(self) -> str | None: ...
    def get_previous_link(self) -> str | None: ...
    def get_html_context(self) -> HtmlContextWithPageLinks: ...
    def get_count(self, queryset: QuerySet | Sequence) -> int: ...

class CursorPagination(BasePagination):
    base_url: str | None
    cursor_query_description: str
    cursor_query_param: str
    cursor: Cursor | None
    has_next: bool | None
    has_previous: bool | None
    invalid_cursor_message: str
    max_page_size: int | None
    next_position: str | None
    offset_cutoff: int
    ordering: str | list[str] | tuple[str, ...]
    page_size_query_description: str
    page_size_query_param: str | None
    page_size: int | None
    page: list[Any] | None
    previous_position: str | None
    template: str
    def paginate_queryset(
        self, queryset: QuerySet[_MT], request: Request, view: APIView | None = ...
    ) -> list[_MT] | None: ...
    def get_page_size(self, request: Request) -> int | None: ...
    def get_next_link(self) -> str | None: ...
    def get_previous_link(self) -> str | None: ...
    def get_html_context(self) -> HtmlContext: ...
    def get_ordering(self, request: Request, queryset: QuerySet, view: APIView) -> tuple[str, ...]: ...
    def decode_cursor(self, request: Request) -> Cursor | None: ...
    def encode_cursor(self, cursor: Cursor) -> str: ...
    def _get_position_from_instance(self, instance: Any, ordering: Sequence[str]) -> str: ...



from typing import Any

from django.db.models import Func
from django.db.models.fields import Field
from django.db.models.fields.json import JSONField

class Cast(Func):
    def __init__(self, expression: Any, output_field: str | Field) -> None: ...

class Coalesce(Func): ...

class Collate(Func):
    def __init__(self, expression: Any, collation: str) -> None: ...

class Greatest(Func): ...

class JSONObject(Func):
    output_field: JSONField

class Least(Func): ...
class NullIf(Func): ...



from collections.abc import Iterator
from typing import Any

from django.core.serializers import base
from django.core.serializers.base import DeserializedObject
from django.db.models.base import Model

class Serializer(base.Serializer):
    objects: list[Any]
    def get_dump_object(self, obj: Model) -> dict[str, Any]: ...

def Deserializer(
    object_list: list[dict[str, Any]], *, using: str = ..., ignorenonexistent: bool = ..., **options: Any
) -> Iterator[DeserializedObject]: ...



from typing import Any

from django.db.models.expressions import Func
from django.db.models.functions.mixins import FixDurationInputMixin, NumericOutputFieldMixin

class Aggregate(Func):
    filter_template: str
    filter: Any
    allow_distinct: bool
    def __init__(self, *expressions: Any, distinct: bool = ..., filter: Any | None = ..., **extra: Any) -> None: ...

class Avg(FixDurationInputMixin, NumericOutputFieldMixin, Aggregate): ...
class Count(Aggregate): ...
class Max(Aggregate): ...
class Min(Aggregate): ...
class StdDev(NumericOutputFieldMixin, Aggregate): ...
class Sum(FixDurationInputMixin, Aggregate): ...
class Variance(NumericOutputFieldMixin, Aggregate): ...



import sys
from _typeshed import FileDescriptorLike
from typing import Any
from typing_extensions import TypeAlias

# Must be a list of length 7, containing 6 ints and a list of NCCS 1-character bytes or ints.
_Attr: TypeAlias = list[int | list[bytes | int]] | list[int | list[bytes]] | list[int | list[int]]
# Same as _Attr for return types; we use Any to avoid a union.
_AttrReturn: TypeAlias = list[Any]

if sys.platform != "win32":
    B0: int
    B1000000: int
    B110: int
    B115200: int
    B1152000: int
    B1200: int
    B134: int
    B150: int
    B1500000: int
    B1800: int
    B19200: int
    B200: int
    B2000000: int
    B230400: int
    B2400: int
    B2500000: int
    B300: int
    B3000000: int
    B3500000: int
    B38400: int
    B4000000: int
    B460800: int
    B4800: int
    B50: int
    B500000: int
    B57600: int
    B576000: int
    B600: int
    B75: int
    B921600: int
    B9600: int
    BRKINT: int
    BS0: int
    BS1: int
    BSDLY: int
    CBAUD: int
    CBAUDEX: int
    CDEL: int
    CDSUSP: int
    CEOF: int
    CEOL: int
    CEOL2: int
    CEOT: int
    CERASE: int
    CESC: int
    CFLUSH: int
    CIBAUD: int
    CINTR: int
    CKILL: int
    CLNEXT: int
    CLOCAL: int
    CNUL: int
    COMMON: int
    CQUIT: int
    CR0: int
    CR1: int
    CR2: int
    CR3: int
    CRDLY: int
    CREAD: int
    CRPRNT: int
    CRTSCTS: int
    CS5: int
    CS6: int
    CS7: int
    CS8: int
    CSIZE: int
    CSTART: int
    CSTOP: int
    CSTOPB: int
    CSUSP: int
    CSWTCH: int
    CWERASE: int
    ECHO: int
    ECHOCTL: int
    ECHOE: int
    ECHOK: int
    ECHOKE: int
    ECHONL: int
    ECHOPRT: int
    EXTA: int
    EXTB: int
    FF0: int
    FF1: int
    FFDLY: int
    FIOASYNC: int
    FIOCLEX: int
    FIONBIO: int
    FIONCLEX: int
    FIONREAD: int
    FLUSHO: int
    HUPCL: int
    IBSHIFT: int
    ICANON: int
    ICRNL: int
    IEXTEN: int
    IGNBRK: int
    IGNCR: int
    IGNPAR: int
    IMAXBEL: int
    INIT_C_CC: int
    INLCR: int
    INPCK: int
    IOCSIZE_MASK: int
    IOCSIZE_SHIFT: int
    ISIG: int
    ISTRIP: int
    IUCLC: int
    IXANY: int
    IXOFF: int
    IXON: int
    N_MOUSE: int
    N_PPP: int
    N_SLIP: int
    N_STRIP: int
    N_TTY: int
    NCC: int
    NCCS: int
    NL0: int
    NL1: int
    NLDLY: int
    NOFLSH: int
    NSWTCH: int
    OCRNL: int
    OFDEL: int
    OFILL: int
    OLCUC: int
    ONLCR: int
    ONLRET: int
    ONOCR: int
    OPOST: int
    PARENB: int
    PARMRK: int
    PARODD: int
    PENDIN: int
    TAB0: int
    TAB1: int
    TAB2: int
    TAB3: int
    TABDLY: int
    TCFLSH: int
    TCGETA: int
    TCGETS: int
    TCIFLUSH: int
    TCIOFF: int
    TCIOFLUSH: int
    TCION: int
    TCOFLUSH: int
    TCOOFF: int
    TCOON: int
    TCSADRAIN: int
    TCSAFLUSH: int
    TCSANOW: int
    TCSASOFT: int
    TCSBRK: int
    TCSBRKP: int
    TCSETA: int
    TCSETAF: int
    TCSETAW: int
    TCSETS: int
    TCSETSF: int
    TCSETSW: int
    TCXONC: int
    TIOCCONS: int
    TIOCEXCL: int
    TIOCGETD: int
    TIOCGICOUNT: int
    TIOCGLCKTRMIOS: int
    TIOCGPGRP: int
    TIOCGSERIAL: int
    TIOCGSIZE: int
    TIOCGSOFTCAR: int
    TIOCGWINSZ: int
    TIOCINQ: int
    TIOCLINUX: int
    TIOCM_CAR: int
    TIOCM_CD: int
    TIOCM_CTS: int
    TIOCM_DSR: int
    TIOCM_DTR: int
    TIOCM_LE: int
    TIOCM_RI: int
    TIOCM_RNG: int
    TIOCM_RTS: int
    TIOCM_SR: int
    TIOCM_ST: int
    TIOCMBIC: int
    TIOCMBIS: int
    TIOCMGET: int
    TIOCMIWAIT: int
    TIOCMSET: int
    TIOCNOTTY: int
    TIOCNXCL: int
    TIOCOUTQ: int
    TIOCPKT_DATA: int
    TIOCPKT_DOSTOP: int
    TIOCPKT_FLUSHREAD: int
    TIOCPKT_FLUSHWRITE: int
    TIOCPKT_NOSTOP: int
    TIOCPKT_START: int
    TIOCPKT_STOP: int
    TIOCPKT: int
    TIOCSCTTY: int
    TIOCSER_TEMT: int
    TIOCSERCONFIG: int
    TIOCSERGETLSR: int
    TIOCSERGETMULTI: int
    TIOCSERGSTRUCT: int
    TIOCSERGWILD: int
    TIOCSERSETMULTI: int
    TIOCSERSWILD: int
    TIOCSETD: int
    TIOCSLCKTRMIOS: int
    TIOCSPGRP: int
    TIOCSSERIAL: int
    TIOCSSIZE: int
    TIOCSSOFTCAR: int
    TIOCSTI: int
    TIOCSWINSZ: int
    TIOCTTYGSTRUCT: int
    TOSTOP: int
    VDISCARD: int
    VEOF: int
    VEOL: int
    VEOL2: int
    VERASE: int
    VINTR: int
    VKILL: int
    VLNEXT: int
    VMIN: int
    VQUIT: int
    VREPRINT: int
    VSTART: int
    VSTOP: int
    VSUSP: int
    VSWTC: int
    VSWTCH: int
    VT0: int
    VT1: int
    VTDLY: int
    VTIME: int
    VWERASE: int
    XCASE: int
    XTABS: int

    def tcgetattr(__fd: FileDescriptorLike) -> _AttrReturn: ...
    def tcsetattr(__fd: FileDescriptorLike, __when: int, __attributes: _Attr) -> None: ...
    def tcsendbreak(__fd: FileDescriptorLike, __duration: int) -> None: ...
    def tcdrain(__fd: FileDescriptorLike) -> None: ...
    def tcflush(__fd: FileDescriptorLike, __queue: int) -> None: ...
    def tcflow(__fd: FileDescriptorLike, __action: int) -> None: ...
    if sys.version_info >= (3, 11):
        def tcgetwinsize(__fd: FileDescriptorLike) -> tuple[int, int]: ...
        def tcsetwinsize(__fd: FileDescriptorLike, __winsize: tuple[int, int]) -> None: ...

    class error(Exception): ...



from typing import Any

class CPointerBase:
    ptr_type: Any
    destructor: Any
    null_ptr_exception_class: Any
    @property
    def ptr(self) -> Any: ...
    @ptr.setter
    def ptr(self, ptr: Any) -> None: ...
    def __del__(self) -> None: ...



from typing import Any

from django.contrib.gis.geos.libgeos import GEOSFuncFactory

class UnaryPredicate(GEOSFuncFactory):
    argtypes: Any
    restype: Any
    errcheck: Any

class BinaryPredicate(UnaryPredicate):
    argtypes: Any

geos_hasz: Any
geos_isclosed: Any
geos_isempty: Any
geos_isring: Any
geos_issimple: Any
geos_isvalid: Any
geos_contains: Any
geos_covers: Any
geos_crosses: Any
geos_disjoint: Any
geos_equals: Any
geos_equalsexact: Any
geos_intersects: Any
geos_overlaps: Any
geos_relatepattern: Any
geos_touches: Any
geos_within: Any



from typing import Any

from django.contrib.gis.db.backends.base.operations import BaseSpatialOperations
from django.contrib.gis.db.backends.utils import SpatialOperator
from django.db.backends.oracle.operations import DatabaseOperations

DEFAULT_TOLERANCE: str

class SDOOperator(SpatialOperator):
    sql_template: str

class SDODWithin(SpatialOperator):
    sql_template: str

class SDODisjoint(SpatialOperator):
    sql_template: Any

class SDORelate(SpatialOperator):
    sql_template: str
    def check_relate_argument(self, arg: Any) -> None: ...

class OracleOperations(BaseSpatialOperations, DatabaseOperations):
    name: str
    oracle: bool
    disallowed_aggregates: Any
    Adapter: Any
    extent: str
    unionagg: str
    function_names: Any
    select: str
    gis_operators: Any
    unsupported_functions: Any
    def geo_quote_name(self, name: Any) -> Any: ...
    def geo_db_type(self, f: Any) -> Any: ...
    def get_distance(self, f: Any, value: Any, lookup_type: Any) -> Any: ...
    def get_geom_placeholder(self, f: Any, value: Any, compiler: Any) -> Any: ...
    def spatial_aggregate_name(self, agg_name: Any) -> Any: ...
    def geometry_columns(self) -> Any: ...
    def spatial_ref_sys(self) -> Any: ...
    def modify_insert_params(self, placeholder: Any, params: Any) -> Any: ...
    def get_geometry_converter(self, expression: Any) -> Any: ...
    def get_area_att_for_field(self, field: Any) -> Any: ...
